@@ -30,14 +30,13 @@ except ImportError:
     __githash__ = ''
 
 # set up the test command
-_test_runner = None
 def _get_test_runner():
     from .tests.helper import TestRunner
     return TestRunner(__path__[0])
 
 def test(package=None, test_path=None, args=None, plugins=None,
          verbose=False, pastebin=None, remote_data=False, pep8=False,
-         pdb=False, coverage=False):
+         pdb=False, coverage=False, open_files=False):
     """
     Run Astropy tests using py.test. A proper set of arguments is
     constructed and passed to `pytest.main`.
@@ -87,6 +86,11 @@ def test(package=None, test_path=None, args=None, plugins=None,
         Generate a test coverage report.  The result will be placed in
         the directory htmlcov.
 
+    open_files : bool, optional
+        Fail when any tests leave files open.  Off by default, because
+        this adds extra run time to the test suite.  Works only on
+        platforms with a working `lsof` command.
+
     See Also
     --------
     pytest.main : py.test function wrapped by `run_tests`.
@@ -97,8 +101,29 @@ def test(package=None, test_path=None, args=None, plugins=None,
         package=package, test_path=test_path, args=args,
         plugins=plugins, verbose=verbose, pastebin=pastebin,
         remote_data=remote_data, pep8=pep8, pdb=pdb,
-        coverage=coverage)
+        coverage=coverage, open_files=open_files)
 
-#if we are *not* in setup mode, import the logger
+# if we are *not* in setup mode, import the logger and possibly populate the
+# configuration file with the defaults
 if not _ASTROPY_SETUP_:
-    from logger import log
+    from .logger import log
+    from . import config
+
+    import os
+    from warnings import warn
+
+    # add these here so we only need to cleanup the namespace at the end
+    config_dir = None
+
+    if not os.environ.get('ASTROPY_SKIP_CONFIG_UPDATE', False):
+        config_dir = os.path.dirname(__file__)
+        try:
+            config.configuration.update_default_config(__package__, config_dir)
+        except config.configuration.ConfigurationDefaultMissingError as e:
+            wmsg = (e.args[0] + " Cannot install default profile. If you are "
+                    "importing from source, this is expected.")
+            warn(config.configuration.ConfigurationDefaultMissingWarning(wmsg))
+            del e
+
+    del os, warn, config_dir  # clean up namespace
+
