@@ -1,4 +1,5 @@
-""" Asciitable: an extensible ASCII table reader and writer.
+# Licensed under a 3-clause BSD style license - see LICENSE.rst
+""" An extensible ASCII table reader and writer.
 
 core.py:
   Core base classes and functions for reading and writing tables.
@@ -6,7 +7,7 @@ core.py:
 :Copyright: Smithsonian Astrophysical Observatory (2010)
 :Author: Tom Aldcroft (aldcroft@head.cfa.harvard.edu)
 """
-## 
+##
 ## Redistribution and use in source and binary forms, with or without
 ## modification, are permitted provided that the following conditions are met:
 ##     * Redistributions of source code must retain the above copyright
@@ -17,7 +18,7 @@ core.py:
 ##     * Neither the name of the Smithsonian Astrophysical Observatory nor the
 ##       names of its contributors may be used to endorse or promote products
 ##       derived from this software without specific prior written permission.
-## 
+##
 ## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ## ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 ## WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
@@ -26,7 +27,7 @@ core.py:
 ## (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 ## LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ## ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-## (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS  
+## (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ## SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 import os
@@ -35,9 +36,10 @@ import re
 import csv
 import itertools
 import numpy
+from contextlib import contextmanager
 
 from ...table import Table
-
+from ...utils.data import get_readable_fileobj
 
 class InconsistentTableError(ValueError):
     pass
@@ -78,33 +80,31 @@ except NameError:
                 return True
         return False
 
-class Keyword(object):
-    """Table keyword"""
-    def __init__(self, name, value, units=None, comment=None, format=None):
-        self.name = name
-        self.value = value
-        self.units = units
-        self.comment = comment
-        self.format = format
 
 class NoType(object):
     pass
 
+
 class StrType(NoType):
     pass
 
+
 class NumType(NoType):
     pass
 
+
 class FloatType(NumType):
     pass
 
+
 class IntType(NumType):
     pass
 
+
 class AllType(StrType, FloatType, IntType):
     pass
 
+
 class Column(object):
     """Table column.
 
@@ -122,20 +122,12 @@ class Column(object):
         self.type = NoType
         self.str_vals = []
         self.fill_values = {}
-        self.formatter = None
-    
-    def __iter__(self):
-        '''iterate over formated column values
-        
-        Each value is paased through self.formatter.
-        If str(self.formatter(value)) is found in the fill_values specification,
-        the corresponding fill_value is returned, otherwise the formated value.
-        '''
-        for val in self.data:
-            yield self.fill_values.get(str(self.formatter(val)).strip(), self.formatter(val))
+
 
 class BaseInputter(object):
-    """Get the lines from the table input and return a list of lines.  The input table can be one of:
+    """
+    Get the lines from the table input and return a list of lines.  The input
+    table can be one of:
 
     * File name
     * String (newline separated) with all header and data lines (must have at least 2 lines)
@@ -144,15 +136,15 @@ class BaseInputter(object):
     """
     def get_lines(self, table):
         """Get the lines from the ``table`` input.
-        
+
         :param table: table input
         :returns: list of lines
         """
         try:
-            if hasattr(table, 'read'):
-                table = table.read()
-            elif '\n' not in table and '\r' not in table + '':
-                table = open(table, 'r').read()
+            if (hasattr(table, 'read') or
+                ('\n' not in table and '\r' not in table + '')):
+                with get_readable_fileobj(table) as file_obj:
+                    table = file_obj.read()
             lines = table.splitlines()
         except TypeError:
             try:
@@ -162,7 +154,8 @@ class BaseInputter(object):
                 iter(table)
                 lines = table
             except TypeError:
-                raise TypeError('Input "table" must be a string (filename or data) or an iterable')
+                raise TypeError(
+                    'Input "table" must be a string (filename or data) or an iterable')
 
         return self.process_lines(lines)
 
@@ -178,6 +171,7 @@ class BaseInputter(object):
         characters if a row is split into lines."""
         return lines
 
+
 class BaseSplitter(object):
     """Base splitter that uses python's split method to do the work.
 
@@ -194,11 +188,11 @@ class BaseSplitter(object):
 
       reader.header.splitter.process_val = lambda x: x.lstrip()
       reader.data.splitter.process_val = None
-      
-    :param delimiter: one-character string used to separate fields 
+
+    :param delimiter: one-character string used to separate fields
     """
     delimiter = None
-    
+
     def process_line(self, line):
         """Remove whitespace at the beginning or end of line.  This is especially useful for
         whitespace-delimited files to prevent spurious columns at the beginning or end."""
@@ -224,7 +218,7 @@ class BaseSplitter(object):
         else:
             delimiter = self.delimiter
         return delimiter.join(str(x) for x in vals)
-    
+
 
 class DefaultSplitter(BaseSplitter):
     """Default class to split strings into columns using python csv.  The class
@@ -233,7 +227,7 @@ class DefaultSplitter(BaseSplitter):
     Typical usage::
 
       # lines = ..
-      splitter = asciitable.DefaultSplitter()
+      splitter = ascii.DefaultSplitter()
       for col_vals in splitter(lines):
           for col_val in col_vals:
                ...
@@ -251,10 +245,10 @@ class DefaultSplitter(BaseSplitter):
     escapechar = None
     quoting = csv.QUOTE_MINIMAL
     skipinitialspace = True
-    
+
     def process_line(self, line):
         """Remove whitespace at the beginning or end of line.  This is especially useful for
-        whitespace-delimited files to prevent spurious columns at the beginning or end. 
+        whitespace-delimited files to prevent spurious columns at the beginning or end.
         If splitting on whitespace then replace unquoted tabs with space first"""
         if self.delimiter == '\s':
             line = _replace_tab_with_space(line, self.escapechar, self.quotechar)
@@ -292,7 +286,7 @@ class DefaultSplitter(BaseSplitter):
                 yield [self.process_val(x) for x in vals]
             else:
                 yield vals
-            
+
     def join(self, vals):
         if self.delimiter is None:
             delimiter = ' '
@@ -310,10 +304,13 @@ class DefaultSplitter(BaseSplitter):
                                          )
         self.csv_writer_out.seek(0)
         self.csv_writer_out.truncate()
-        self.csv_writer.writerow(vals)        
+        if self.process_val:
+            vals = [self.process_val(x) for x in vals]
+        self.csv_writer.writerow(vals)
 
         return self.csv_writer_out.getvalue()
-    
+
+
 def _replace_tab_with_space(line, escapechar, quotechar):
     """Replace tab with space within ``line`` while respecting quoted substrings"""
     newline = []
@@ -328,12 +325,24 @@ def _replace_tab_with_space(line, escapechar, quotechar):
         newline.append(char)
     return ''.join(newline)
 
+
 def _get_line_index(line_or_func, lines):
+    """Return the appropriate line index, depending on ``line_or_func`` which
+    can be either a function, a positive or negative int, or None.
+    """
+
     if hasattr(line_or_func, '__call__'):
         return line_or_func(lines)
+    elif line_or_func:
+        if line_or_func >= 0:
+            return line_or_func
+        else:
+            n_lines = sum(1 for line in lines)
+            return n_lines + line_or_func
     else:
         return line_or_func
 
+
 class BaseHeader(object):
     """Base table header reader
 
@@ -352,11 +361,11 @@ class BaseHeader(object):
     names = None
     include_names = None
     exclude_names = None
-    write_spacer_lines = ['ASCIITABLE_WRITE_SPACER_LINE']
+    write_spacer_lines = ['ASCII_TABLE_WRITE_SPACER_LINE']
 
     def __init__(self):
         self.splitter = self.__class__.splitter_class()
-       
+
     def _set_cols_from_names(self):
         # Filter full list of non-null column names with the include/exclude lists
         names = set(self.names)
@@ -364,7 +373,7 @@ class BaseHeader(object):
             names.intersection_update(self.include_names)
         if self.exclude_names is not None:
             names.difference_update(self.exclude_names)
-            
+
         self.cols = [Column(name=x, index=i) for i, x in enumerate(self.names) if x in names]
 
     def get_cols(self, lines):
@@ -387,7 +396,8 @@ class BaseHeader(object):
                 try:
                     first_data_vals = next(self.data.get_str_vals())
                 except StopIteration:
-                    raise InconsistentTableError('No data lines found so cannot autogenerate column names')
+                    raise InconsistentTableError('No data lines found so cannot autogenerate '
+                                                 'column names')
                 n_data_cols = len(first_data_vals)
                 self.names = [self.auto_format % i for i in range(1, n_data_cols+1)]
 
@@ -400,7 +410,7 @@ class BaseHeader(object):
                 raise ValueError('No header line found in table')
 
             self.names = next(self.splitter([line]))
-        
+
         self._set_cols_from_names()
 
     def process_lines(self, lines):
@@ -464,13 +474,12 @@ class BaseData(object):
     end_line = None
     comment = None
     splitter_class = DefaultSplitter
-    write_spacer_lines = ['ASCIITABLE_WRITE_SPACER_LINE']
+    write_spacer_lines = ['ASCII_TABLE_WRITE_SPACER_LINE']
     formats = {}
-    default_formatter = str
     fill_values = []
     fill_include_names = None
     fill_exclude_names = None
-    
+
     def __init__(self):
         self.splitter = self.__class__.splitter_class()
 
@@ -506,7 +515,7 @@ class BaseData(object):
 
     def masks(self, cols):
         """Set fill value for each column and then apply that fill value
-        
+
         In the first step it is evaluated with value from ``fill_values`` applies to
         which column using ``fill_include_names`` and ``fill_exclude_names``.
         In the second step all replacements are done for the appropriate columns.
@@ -514,7 +523,7 @@ class BaseData(object):
         if self.fill_values:
             self._set_fill_values(cols)
             self._set_masks(cols)
-    
+
     def _set_fill_values(self, cols):
         """Set the fill values of the individual cols based on fill_values of BaseData
 
@@ -527,7 +536,7 @@ class BaseData(object):
             #if input is only one <fill_spec>, then make it a list
             try:
                 self.fill_values[0] + ''
-                self.fill_values = [ self.fill_values ] 
+                self.fill_values = [self.fill_values]
             except TypeError:
                 pass
             # Step 1: Set the default list of columns which are affected by fill_values
@@ -536,26 +545,29 @@ class BaseData(object):
                 colnames.intersection_update(self.fill_include_names)
             if self.fill_exclude_names is not None:
                 colnames.difference_update(self.fill_exclude_names)
-        
+
             # Step 2a: Find out which columns are affected by this tuple
-            # iterate over reversed order, so last condition is set first and overwritten by earlier conditions
+            # iterate over reversed order, so last condition is set first and
+            # overwritten by earlier conditions
             for replacement in reversed(self.fill_values):
                 if len(replacement) < 2:
-                    raise ValueError("Format of fill_values must be (<bad>, <fill>, <optional col1>, ...)")
+                    raise ValueError("Format of fill_values must be "
+                                     "(<bad>, <fill>, <optional col1>, ...)")
                 elif len(replacement) == 2:
                     affect_cols = colnames
                 else:
                     affect_cols = replacement[2:]
-                
+
                 for i, key in ((i, x) for i, x in enumerate(self.header.colnames) if x in affect_cols):
                     cols[i].fill_values[replacement[0]] = str(replacement[1])
 
     def _set_masks(self, cols):
-        """Replace string values in col.str_vales and set masks"""
+        """Replace string values in col.str_vals and set masks"""
         if self.fill_values:
             for col in (col for col in cols if col.fill_values):
-                col.mask = [False] * len(col.str_vals)
-                for i, str_val in ((i, x) for i, x in enumerate(col.str_vals) if x in col.fill_values):
+                col.mask = numpy.zeros(len(col.str_vals), dtype=numpy.bool)
+                for i, str_val in ((i, x) for i, x in enumerate(col.str_vals)
+                                   if x in col.fill_values):
                     col.str_vals[i] = col.fill_values[str_val]
                     col.mask[i] = True
 
@@ -568,28 +580,33 @@ class BaseData(object):
         while len(lines) < data_start_line:
             lines.append(itertools.cycle(self.write_spacer_lines))
 
-        formatters = []
-        for col in self.cols:
-            formatter = self.formats.get(col.name, self.default_formatter)
-            if not hasattr(formatter, '__call__'):
-                formatter = _format_func(formatter)
-            col.formatter = formatter
+        with self._set_col_formats(self.cols, self.formats):
+            col_str_iters = [col.iter_str_vals() for col in self.cols]
+            for vals in izip(*col_str_iters):
+                lines.append(self.splitter.join(vals))
 
-        for vals in izip(*self.cols):
-            lines.append(self.splitter.join(vals))
+    @contextmanager
+    def _set_col_formats(self, cols, formats):
+        """
+        Context manager to save the internal column formats in `cols` and
+        override with any custom `formats`.
+        """
+        orig_formats = [col.format for col in cols]
+        for col in cols:
+            if col.name in formats:
+                col.format = formats[col.name]
 
+        yield  # execute the nested context manager block
 
-def _format_func(format_str):
-    def func(val):
-        return format_str % val
-    return func
-
+        # Restore the original column format values
+        for col, orig_format in izip(cols, orig_formats):
+            col.format = orig_format
 
 class DictLikeNumpy(dict):
     """Provide minimal compatibility with numpy rec array API for BaseOutputter
     object::
-      
-      table = asciitable.read('mytable.dat', numpy=False)
+
+      table = ascii.read('mytable.dat', numpy=False)
       table.field('x')    # List of elements in column 'x'
       table.dtype.names   # get column names in order
       table[1]            # returns row 1 as a list
@@ -648,7 +665,7 @@ def convert_numpy(numpy_type):
     float, str) that is produced by the converter function.
     """
 
-    # Infer converter type from an instance of numpy_type. 
+    # Infer converter type from an instance of numpy_type.
     type_name = numpy.array([], dtype=numpy_type).dtype.name
     if 'int' in type_name:
         converter_type = IntType
@@ -658,7 +675,7 @@ def convert_numpy(numpy_type):
         converter_type = StrType
     else:
         converter_type = AllType
-    
+
     def converter(vals):
         return numpy.array(vals, numpy_type)
     return converter, converter_type
@@ -683,7 +700,7 @@ class BaseOutputter(object):
                     raise ValueError()
                 if issubclass(converter_type, col.type):
                     converters_out.append((converter_func, converter_type))
-                    
+
         except (ValueError, TypeError):
             raise ValueError('Error: invalid format for converters, see documentation\n%s' %
                              converters)
@@ -707,55 +724,6 @@ class BaseOutputter(object):
                 except IndexError:
                     raise ValueError('Column %s failed to convert' % col.name)
 
-class NumpyOutputter(BaseOutputter):
-    """Output the table as a numpy.rec.recarray
-
-    Missing or bad data values are handled at two levels.  The first is in
-    the data reading step where if ``data.fill_values`` is set then any
-    occurences of a bad value are replaced by the correspond fill value.
-    At the same time a boolean list ``mask`` is created in the column object.
-
-    The second stage is when converting to numpy arrays which by default generates
-    masked arrays, if ``data.fill_values`` is set and plain arrays if it is not.
-    In the rare case that plain arrays are needed set ``auto_masked`` (default = True) and
-    ``default_masked`` (default = False) to control this behavior as follows:
-
-    ===========  ==============  ===========  ============
-    auto_masked  default_masked  fill_values  output
-    ===========  ==============  ===========  ============
-    --            True           --           masked_array
-    --            False          None         array
-    True          --             dict(..)     masked_array
-    False         --             dict(..)     array
-    ===========  ==============  ===========  ============
-
-    To set these values use::
-    
-      Outputter = asciitable.NumpyOutputter()
-      Outputter.default_masked = True
-    
-    """
-
-    auto_masked_array = True
-    default_masked_array = False
-
-    default_converters = [convert_numpy(numpy.int),
-                          convert_numpy(numpy.float),
-                          convert_numpy(numpy.str)]
-
-    def __call__(self, cols):
-        self._convert_vals(cols)
-        recarr = numpy.rec.fromarrays([x.data for x in cols], names=[x.name for x in cols])
-        if self.default_masked_array or (self.auto_masked_array and
-                                         any(col.fill_values for col in cols)):
-            maarr = recarr.view(numpy.ma.MaskedArray)
-            for col in cols:
-                if col.fill_values:
-                    maarr[col.name] = numpy.ma.masked_where(col.mask, maarr[col.name])
-            return maarr
-        else:
-            return recarr
-
 
 class TableOutputter(BaseOutputter):
     """Output the table as an astropy.table.Table object.
@@ -771,8 +739,19 @@ class TableOutputter(BaseOutputter):
 
     def __call__(self, cols):
         self._convert_vals(cols)
-        out = Table([x.data for x in cols], names=[x.name for x in cols])
-        # To Do: add support for column and table metadata
+
+        # If there are any values that were filled and tagged with a mask bit then this
+        # will be a masked table.  Otherwise use a plain table.
+        masked = any(hasattr(col, 'mask') and numpy.any(col.mask) for col in cols)
+
+        out = Table([x.data for x in cols], names=[x.name for x in cols], masked=masked)
+        for col, out_col in zip(cols, out.columns.values()):
+            if masked and hasattr(col, 'mask'):
+                out_col.data.mask = col.mask
+            for attr in ('format', 'units', 'description'):
+                if hasattr(col, attr):
+                    setattr(out_col, attr, getattr(col, attr))
+
         return out
 
 
@@ -794,8 +773,7 @@ class BaseReader(object):
         self.data = BaseData()
         self.inputter = BaseInputter()
         self.outputter = TableOutputter()
-        self.meta = {}                  # Placeholder for storing table metadata 
-        self.keywords = []              # Placeholder for storing table Keywords
+        self.meta = {}                  # Placeholder for storing table metadata
         # Data and Header instances benefit from a little cross-coupling.  Header may need to
         # know about number of data columns for auto-column name generation and Data may
         # need to know about header (e.g. for fixed-width tables where widths are spec'd in header.
@@ -826,7 +804,7 @@ class BaseReader(object):
         except TypeError:
             # Not a string.
             pass
-            
+
         # Same from __init__.  ??? Do these need to be here?
         self.data.header = self.header
         self.header.data = self.data
@@ -839,13 +817,13 @@ class BaseReader(object):
         self.data.splitter.cols = cols
 
         for i, str_vals in enumerate(self.data.get_str_vals()):
-            if len(str_vals) != n_data_cols:                
+            if len(str_vals) != n_data_cols:
                 str_vals = self.inconsistent_handler(str_vals, n_data_cols)
-                
+
                 #if str_vals is None, we skip this row
                 if str_vals is None:
                     continue
-                
+
                 #otherwise, we raise an error only if it is still inconsistent
                 if len(str_vals) != n_data_cols:
                     errmsg = ('Number of header columns (%d) inconsistent with '
@@ -859,23 +837,23 @@ class BaseReader(object):
                 col.str_vals.append(str_vals[col.index])
 
         self.data.masks(cols)
-        self.table = self.outputter(cols)
+        table = self.outputter(cols)
         self.cols = self.header.cols
 
-        return self.table
-    
+        return table
+
     def inconsistent_handler(self, str_vals, ncols):
         """Adjust or skip data entries if a row is inconsistent with the header.
-        
+
         The default implementation does no adjustment, and hence will always trigger
-        an exception in read() any time the number of data entries does not match 
+        an exception in read() any time the number of data entries does not match
         the header.
-        
+
         Note that this will *not* be called if the row already matches the header.
 
         :param str_vals: A list of value strings from the current row of the table.
         :param ncols: The expected number of entries from the table header.
-        :returns: 
+        :returns:
             list of strings to be parsed into data entries in the output table. If
             the length of this list does not match ``ncols``, an exception will be
             raised in read().  Can also be None, in which case the row will be
@@ -896,21 +874,17 @@ class BaseReader(object):
             comment_lines = []
         return comment_lines
 
-    def write(self, table=None):
+    def write(self, table):
         """Write ``table`` as list of strings.
 
-        :param table: asciitable Reader object
+        :param table: input table data (astropy.table.Table object)
         :returns: list of strings corresponding to ASCII table
         """
-        if table is None:
-            table = self
-
         # link information about the columns to the writer object (i.e. self)
         self.header.cols = table.cols
-        self.data.cols = self.header.cols
-        self.data.masks(self.data.cols)
+        self.data.cols = table.cols
 
-        # Write header and data to lines list 
+        # Write header and data to lines list
         lines = []
         self.header.write(lines)
         self.data.write(lines)
@@ -929,16 +903,22 @@ class ContinuationLinesInputter(BaseInputter):
     """
 
     continuation_char = '\\'
+    replace_char = ' '
+    # If no_continue is not None then lines matching this regex are not subject
+    # to line continuation.  The initial use case here is Daophot.  In this
+    # case the continuation character is just replaced with replace_char.
+    no_continue = None
 
     def process_lines(self, lines):
-        striplines = (x.strip() for x in lines)
-        lines = [x for x in striplines if len(x) > 0]
+        re_no_continue = re.compile(self.no_continue) if self.no_continue else None
 
         parts = []
         outlines = []
         for line in lines:
+            if re_no_continue and re_no_continue.match(line):
+                line = line.replace(self.continuation_char, self.replace_char)
             if line.endswith(self.continuation_char):
-                parts.append(line.rstrip(self.continuation_char))
+                parts.append(line.replace(self.continuation_char, self.replace_char))
             else:
                 parts.append(line)
                 outlines.append(''.join(parts))
@@ -954,7 +934,7 @@ class WhitespaceSplitter(DefaultSplitter):
         in_quote = False
         lastchar = None
         for char in line:
-            if char == self.quotechar and (self.escapechar is None or 
+            if char == self.quotechar and (self.escapechar is None or
                                            lastchar != self.escapechar):
                 in_quote = not in_quote
             if char == '\t' and not in_quote:
@@ -964,7 +944,7 @@ class WhitespaceSplitter(DefaultSplitter):
 
         return ''.join(newline)
 
-extra_reader_pars = ('Reader', 'Inputter', 'Outputter', 
+extra_reader_pars = ('Reader', 'Inputter', 'Outputter',
                      'delimiter', 'comment', 'quotechar', 'header_start',
                      'data_start', 'data_end', 'converters',
                      'data_Splitter', 'header_Splitter',
@@ -1023,8 +1003,8 @@ def _get_reader(Reader, Inputter=None, Outputter=None, **kwargs):
 
     return reader
 
-extra_writer_pars = ('delimiter', 'comment', 'quotechar', 'formats',
-                     'names', 'include_names', 'exclude_names', 
+extra_writer_pars = ('delimiter', 'comment', 'quotechar', 'formats', 'strip_whitespace',
+                     'names', 'include_names', 'exclude_names',
                      'fill_values', 'fill_include_names',
                      'fill_exclude_names')
 
@@ -1047,6 +1027,16 @@ def _get_writer(Writer, **kwargs):
         writer.data.splitter.quotechar = kwargs['quotechar']
     if 'formats' in kwargs:
         writer.data.formats = kwargs['formats']
+    if 'strip_whitespace' in kwargs:
+        if kwargs['strip_whitespace']:
+            # Restore the default SplitterClass process_val method which strips
+            # whitespace.  This may have been changed in the Writer
+            # initialization (e.g. Rdb and Tab)
+            Class = writer.data.splitter.__class__
+            obj = writer.data.splitter
+            writer.data.splitter.process_val = Class.process_val.__get__(obj, Class)
+        else:
+            writer.data.splitter.process_val = None
     if 'names' in kwargs:
         writer.header.names = kwargs['names']
     if 'include_names' in kwargs:
@@ -1060,5 +1050,3 @@ def _get_writer(Writer, **kwargs):
     if 'fill_exclude_names' in kwargs:
         writer.data.fill_exclude_names = kwargs['fill_exclude_names']
     return writer
-
-    
