@@ -24,7 +24,9 @@ FIX_FP_TABLE = maketrans('de', 'DE')
 FIX_FP_TABLE2 = maketrans('dD', 'eE')
 
 
-BLANK_CARD = ' ' * 80
+CARD_LENGTH = 80
+BLANK_CARD = ' ' * CARD_LENGTH
+KEYWORD_LENGTH = 8  # The max length for FITS-standard keywords
 
 
 class Undefined:
@@ -229,11 +231,11 @@ class CardList(list):
 
     @deprecated('3.1', alternative=':meth:`Header.index`')
     def index(self, card):
-        return self._header.cards.index(card)
+        return self._header._cards.index(card)
 
     @deprecated('3.1', alternative=':meth:`Header.count`')
     def count(self, card):
-        return self._header.cards.count(card)
+        return self._header._cards.count(card)
 
     @deprecated('3.1', alternative=':meth:`Header.index`', pending=False)
     def index_of(self, key, backward=False):
@@ -299,10 +301,10 @@ class CardList(list):
 
 
 class Card(_Verify):
-    length = 80
+    length = CARD_LENGTH
 
     # String for a FITS standard compliant (FSC) keyword.
-    _keywd_FSC_RE = re.compile(r'^[A-Z0-9_-]{0,8}$')
+    _keywd_FSC_RE = re.compile(r'^[A-Z0-9_-]{0,%d}$' % KEYWORD_LENGTH)
     # This will match any printable ASCII character excluding '='
     _keywd_hierarch_RE = re.compile(r'^(?:HIERARCH )?(?:^[ -<>-~]+ ?)+$', re.I)
 
@@ -382,13 +384,13 @@ class Card(_Verify):
     _rvkc_identifier = r'[a-zA-Z_]\w*'
     _rvkc_field = _rvkc_identifier + r'(\.\d+)?'
     _rvkc_field_specifier_s = r'%s(\.%s)*' % ((_rvkc_field,) * 2)
-    _rvkc_field_specifier_val = (r'(?P<keyword>%s): (?P<val>%s\s*)' %
+    _rvkc_field_specifier_val = (r'(?P<keyword>%s): (?P<val>%s)' %
                                  (_rvkc_field_specifier_s, _numr_FSC))
-    _rvkc_keyword_val = r'\'%s\'' % _rvkc_field_specifier_val
+    _rvkc_keyword_val = r'\'(?P<rawval>%s)\'' % _rvkc_field_specifier_val
     _rvkc_keyword_val_comm = (r' +%s *(/ *(?P<comm>[ -~]*))?$' %
                               _rvkc_keyword_val)
 
-    _rvkc_field_specifier_val_RE = re.compile(_rvkc_field_specifier_val)
+    _rvkc_field_specifier_val_RE = re.compile(_rvkc_field_specifier_val + '$')
 
     # regular expression to extract the key and the field specifier from a
     # string that is being used to index into a card list that contains
@@ -412,6 +414,7 @@ class Card(_Verify):
         self._keyword = None
         self._value = None
         self._comment = None
+        self._rawvalue = None
         self._image = None
 
         # This attribute is set to False when creating the card from a card
@@ -479,7 +482,8 @@ class Card(_Verify):
             # should be strictly disallowed.
             keyword = keyword.rstrip()
             keyword_upper = keyword.upper()
-            if len(keyword) <= 8 and self._keywd_FSC_RE.match(keyword_upper):
+            if (len(keyword) <= KEYWORD_LENGTH and
+                self._keywd_FSC_RE.match(keyword_upper)):
                 # For keywords with length > 8 they will be HIERARCH cards,
                 # and can have arbitrary case keywords
                 if keyword_upper == 'END':
@@ -596,6 +600,12 @@ class Card(_Verify):
             raise AttributeError('Values cannot be deleted from record-valued '
                                  'keyword cards')
 
+    @property
+    def rawvalue(self):
+        if self._rawvalue is None and self.value:
+            self._rawvalue = self.value
+        return self._rawvalue
+
     @property
     def comment(self):
         """Get the comment attribute from the card image if not already set."""
@@ -774,6 +784,8 @@ class Card(_Verify):
             if eq_idx < 0 or eq_idx > 9:
                 return False
             keyword, rest = image.split('=', 1)
+            if keyword in self._commentary_keywords:
+                return False
             match = self._rvkc_keyword_val_comm_RE.match(rest)
             if match:
                 field_specifier = match.group('keyword')
@@ -781,11 +793,14 @@ class Card(_Verify):
                                           field_specifier))
                 self._field_specifier = field_specifier
                 self._value = _int_or_float(match.group('val'))
+                self._rawvalue = match.group('rawval')
                 return True
         elif len(args) == 2:
             keyword, value = args
             if not isinstance(keyword, basestring):
                 return False
+            if keyword in self._commentary_keywords:
+                return False
             match = self._rvkc_keyword_name_RE.match(keyword)
             if match and isinstance(value, (int, float)):
                 field_specifier = match.group('field_specifier')
@@ -802,19 +817,20 @@ class Card(_Verify):
                                               field_specifier))
                     self._field_specifier = field_specifier
                     self._value = _int_or_float(match.group('val'))
+                    self._rawvalue = value
                     return True
 
     def _parse_keyword(self):
         if self._check_if_rvkc(self._image):
             return self._keyword
 
-        keyword = self._image[:8].strip()
+        keyword = self._image[:KEYWORD_LENGTH].strip()
         keyword_upper = keyword.upper()
         value_indicator = self._image.find('= ')
 
         special = self._commentary_keywords + ['CONTINUE']
 
-        if keyword_upper in special or 0 <= value_indicator <= 8:
+        if keyword_upper in special or 0 <= value_indicator <= KEYWORD_LENGTH:
             # The value indicator should appear in byte 8, but we are flexible
             # and allow this to be fixed
             if value_indicator >= 0:
@@ -829,6 +845,7 @@ class Card(_Verify):
             # This is valid HIERARCH card as described by the HIERARCH keyword
             # convention:
             # http://fits.gsfc.nasa.gov/registry/hierarch_keyword.html
+            self._hierarch = True
             return self._image.split('=', 1)[0][9:].rstrip()
         else:
             warnings.warn('The following header keyword is invalid or follows '
@@ -843,7 +860,7 @@ class Card(_Verify):
         # for commentary cards, no need to parse further
         # Likewise for invalid cards
         if self.keyword.upper() in self._commentary_keywords or self._invalid:
-            return self._image[8:].rstrip()
+            return self._image[KEYWORD_LENGTH:].rstrip()
 
         if self._check_if_rvkc(self._image):
             return self._value
@@ -951,8 +968,8 @@ class Card(_Verify):
             # The equal sign may not be any higher than column 10; anything
             # past that must be considered part of the card value
             if delim_index is None:
-                keyword = image[:8]
-                valuecomment = image[8:]
+                keyword = image[:KEYWORD_LENGTH]
+                valuecomment = image[KEYWORD_LENGTH:]
             elif delim_index > 10 and image[:9] != 'HIERARCH ':
                 keyword = image[:8]
                 valuecomment = image[8:]
@@ -1011,13 +1028,13 @@ class Card(_Verify):
     def _format_keyword(self):
         if self.keyword:
             if self.field_specifier:
-                return '%-8s' % self.keyword.split('.', 1)[0]
+                return '%-*s' % (KEYWORD_LENGTH, self.keyword.split('.', 1)[0])
             elif self._hierarch:
                 return 'HIERARCH %s ' % self.keyword
             else:
-                return '%-8s' % self.keyword
+                return '%-*s' % (KEYWORD_LENGTH, self.keyword)
         else:
-            return ' ' * 8
+            return ' ' * KEYWORD_LENGTH
 
     def _format_value(self):
         # value string
@@ -1044,7 +1061,7 @@ class Card(_Verify):
             value = _format_value(value)
 
         # For HIERARCH cards the value should be shortened to conserve space
-        if not self.field_specifier and len(self.keyword) > 8:
+        if not self.field_specifier and len(self.keyword) > KEYWORD_LENGTH:
             value = value.strip()
 
         return value
@@ -1124,7 +1141,7 @@ class Card(_Verify):
         words = _words_group(value, value_length)
         for idx, word in enumerate(words):
             if idx == 0:
-                headstr = '%-8s= ' % self.keyword
+                headstr = '%-*s= ' % (KEYWORD_LENGTH, self.keyword)
             else:
                 headstr = 'CONTINUE  '
             value = value_format % word
@@ -1147,7 +1164,7 @@ class Card(_Verify):
         same type.
         """
 
-        maxlen = Card.length - len(self.keyword) - 1
+        maxlen = Card.length - KEYWORD_LENGTH
         value = self._format_value()
         output = []
         idx = 0
