@@ -16,8 +16,8 @@ from .table import _TableBaseHDU
 from ..file import PYTHON_MODES, _File
 from ..header import BLOCK_SIZE, _pad_length
 from ..util import (_is_int, _tmp_name, isfile, fileobj_name, fileobj_closed,
-                    fileobj_mode, ignore_sigint, _get_array_memmap, indent)
-from ..verify import _Verify, _ErrList, VerifyError
+                    fileobj_mode, ignore_sigint, _get_array_mmap, indent)
+from ..verify import _Verify, _ErrList, VerifyError, VerifyWarning
 
 
 def fitsopen(name, mode='readonly', memmap=None, save_backup=False, **kwargs):
@@ -491,7 +491,9 @@ class HDUList(list, _Verify):
             name = hdu.name
             if isinstance(name, str):
                 name = name.strip().upper()
-            if name == _key and (_ver is None or _ver == hdu._extver):
+            # 'PRIMARY' should always work as a reference to the first HDU
+            if ((name == _key or (_key == 'PRIMARY' and idx == 0)) and
+                (_ver is None or _ver == hdu._extver)):
                 found = idx
                 nfound += 1
 
@@ -558,7 +560,7 @@ class HDUList(list, _Verify):
             for hdu in self:
                 if verbose:
                     try:
-                        extver = str(hdu.header['extver'])
+                        extver = str(hdu._header['extver'])
                     except KeyError:
                         extver = ''
 
@@ -910,7 +912,7 @@ class HDUList(list, _Verify):
 
             # reset the modification attributes after updating
             for hdu in self:
-                hdu.header._modified = False
+                hdu._header._modified = False
         finally:
             for hdu in self:
                 hdu._postwriteto()
@@ -943,7 +945,7 @@ class HDUList(list, _Verify):
             if sys.platform.startswith('win'):
                 # Collect a list of open mmaps to the data; this well be used
                 # later.  See below.
-                mmaps = [(idx, _get_array_memmap(hdu.data), hdu.data)
+                mmaps = [(idx, _get_array_mmap(hdu.data), hdu.data)
                          for idx, hdu in enumerate(self) if hdu._data_loaded]
 
             hdulist.__file.close()
@@ -953,9 +955,9 @@ class HDUList(list, _Verify):
                 # Close all open mmaps to the data.  This is only necessary on
                 # Windows, which will not allow a file to be renamed or deleted
                 # until all handles to that file have been closed.
-                for idx, map, arr in mmaps:
-                    if map is not None:
-                        map.base.close()
+                for idx, mmap, arr in mmaps:
+                    if mmap is not None:
+                        mmap.close()
 
             os.remove(self.__file.name)
 
@@ -976,7 +978,7 @@ class HDUList(list, _Verify):
                 # Need to update the _file attribute and close any open mmaps
                 # on each HDU
                 if (hdu._data_loaded and
-                    _get_array_memmap(hdu.data) is not None):
+                    _get_array_mmap(hdu.data) is not None):
                     del hdu.data
                 hdu._file = ffo
 
@@ -1021,7 +1023,7 @@ class HDUList(list, _Verify):
         self._resize = False
         self._truncate = False
         for hdu in self:
-            hdu.header._modified = False
+            hdu._header._modified = False
             hdu._new = False
             hdu._file = ffo
 
@@ -1038,7 +1040,7 @@ class HDUList(list, _Verify):
             # determine if any of the HDU is resized
             for hdu in self:
                 # Header:
-                nbytes = len(str(hdu.header))
+                nbytes = len(str(hdu._header))
                 if nbytes != (hdu._datLoc - hdu._hdrLoc):
                     self._resize = True
                     self._truncate = False
@@ -1050,7 +1052,7 @@ class HDUList(list, _Verify):
                 if not hdu._data_loaded or hdu.data is None:
                     continue
 
-                nbytes = hdu.data.nbytes
+                nbytes = hdu.size
                 nbytes = nbytes + _pad_length(nbytes)
                 if nbytes != hdu._datSpan:
                     self._resize = True
