@@ -10,6 +10,7 @@ import warnings
 from contextlib import contextmanager
 
 from .config import ConfigurationItem
+from . import config
 from .utils.compat import inspect_getmodule
 from .utils.console import color_print
 from .utils.misc import find_current_module
@@ -56,8 +57,10 @@ LOG_TO_FILE = ConfigurationItem('log_to_file', True,
                                 "Whether to always log messages to a log "
                                 "file")
 
-LOG_FILE_PATH = ConfigurationItem('log_file_path', '~/.astropy/astropy.log',
-                                  "The file to log messages to")
+LOG_FILE_PATH = ConfigurationItem('log_file_path', '',
+                                  "The file to log messages to. When '', "
+                                  "it defaults to a file 'astropy.log' in "
+                                  "the astropy config directory.")
 
 LOG_FILE_LEVEL = ConfigurationItem('log_file_level', 'INFO',
                                    "Threshold for logging messages to "
@@ -193,9 +196,9 @@ class AstropyLogger(Logger):
                     break
 
         if mod_name is not None:
-            self.warn(message, extra={'origin': mod_name})
+            self.warning(message, extra={'origin': mod_name})
         else:
-            self.warn(message)
+            self.warning(message)
 
     def warnings_logging_enabled(self):
         return self._showwarning_orig is not None
@@ -286,10 +289,11 @@ class AstropyLogger(Logger):
         except NameError:
             ip = None
 
+        if self.exception_logging_enabled():
+            raise LoggingError("Exception logging has already been enabled")
+
         if ip is None:
             #standard python interpreter
-            if self.exception_logging_enabled():
-                raise LoggingError("Exception logging has already been enabled")
             self._excepthook_orig = sys.excepthook
             sys.excepthook = self._excepthook
         else:
@@ -326,10 +330,11 @@ class AstropyLogger(Logger):
         except NameError:
             ip = None
 
+        if not self.exception_logging_enabled():
+            raise LoggingError("Exception logging has not been enabled")
+
         if ip is None:
             #standard python interpreter
-            if not self.exception_logging_enabled():
-                raise LoggingError("Exception logging has not been enabled")
             if sys.excepthook != self._excepthook:
                 raise LoggingError("Cannot disable exception logging: "
                                    "sys.excepthook was not set by this logger, "
@@ -338,11 +343,7 @@ class AstropyLogger(Logger):
             self._excepthook_orig = None
         else:
             #IPython has its own way of dealing with exceptions
-            if not self.exception_logging_enabled():
-                raise LoggingError("Astropy exception logging has not been "
-                                   "enabled in this IPython session")
-            else:
-                ip.set_custom_exc(tuple(), None)
+            ip.set_custom_exc(tuple(), None)
 
     def enable_color(self):
         '''
@@ -375,7 +376,7 @@ class AstropyLogger(Logger):
     @contextmanager
     def log_to_file(self, filename, filter_level=None, filter_origin=None):
         '''
-        Context manager to temporarily log messages to a file
+        Context manager to temporarily log messages to a file.
 
         Parameters
         ----------
@@ -418,12 +419,13 @@ class AstropyLogger(Logger):
         fh.setFormatter(f)
         self.addHandler(fh)
         yield
+        fh.close()
         self.removeHandler(fh)
 
     @contextmanager
     def log_to_list(self, filter_level=None, filter_origin=None):
         '''
-        Context manager to temporarily log messages to a list
+        Context manager to temporarily log messages to a list.
 
         Parameters
         ----------
@@ -498,11 +500,28 @@ class AstropyLogger(Logger):
 
         # Set up the main log file handler if requested (but this might fail if
         # configuration directory or log file is not writeable).
-        if LOG_TO_FILE() and not os.environ.get('ASTROPY_TESTS_RUNNING'):
+        if LOG_TO_FILE():
+            log_file_path = LOG_FILE_PATH()
+
+            # "None" as a string because it comes from config
             try:
-                fh = FileHandler(os.path.expanduser(LOG_FILE_PATH()))
-            except IOError:
-                pass
+                _ASTROPY_TEST_
+                testing_mode = True
+            except NameError:
+                testing_mode = False
+
+            try:
+                if log_file_path == '' or testing_mode:
+                    log_file_path = os.path.join(
+                        config.get_config_dir(), "astropy.log")
+                else:
+                    log_file_path = os.path.expanduser(log_file_path)
+
+                fh = FileHandler(log_file_path)
+            except (IOError, OSError) as e:
+                warnings.warn(
+                    'log file {0!r} could not be opened for writing: '
+                    '{1}'.format(log_file_path, unicode(e)), RuntimeWarning)
             else:
                 formatter = logging.Formatter(LOG_FILE_FORMAT())
                 fh.setFormatter(formatter)
@@ -515,6 +534,7 @@ class AstropyLogger(Logger):
         if LOG_EXCEPTIONS():
             self.enable_exception_logging()
 
+
 # Set up the class and initialize logger
 _orig_logger_cls = logging.getLoggerClass()
 logging.setLoggerClass(AstropyLogger)
