@@ -8,10 +8,10 @@ just `numpy.ndarray` objects, because it provides metadata that cannot
 be easily provided by a single array.
 """
 
-from astropy import setup_helpers
-
 from .nddata import *
+from .nduncertainty import *
+from .flag_collection import *
 
-if not setup_helpers.is_in_build_mode():
+if not _ASTROPY_SETUP_:
     from .convolution.convolve import convolve,convolve_fft
     from .convolution.make_kernel import make_kernel
