@@ -1,3 +1,4 @@
+# Licensed under a 3-clause BSD style license - see LICENSE.rst
 import numpy as np
 
 from astropy.tests.helper import pytest
@@ -9,18 +10,6 @@ from numpy.testing import assert_array_almost_equal_nulp
 
 import itertools
 
-try:
-    import scipy.fftpack
-    HAS_SCIPY = True
-except ImportError:
-    HAS_SCIPY = False
-
-try:
-    import fftw3
-    HAS_FFTW = True
-except ImportError:
-    HAS_FFTW = False
-
 VALID_DTYPES = []
 for dtype_array in ['>f4', '<f4', '>f8', '<f8']:
     for dtype_kernel in ['>f4', '<f4', '>f8', '<f8']:
@@ -40,23 +29,18 @@ Convolved with [0,1] = [0, 1, 2, 3, 4]
 """
 
 # NOTE: use_numpy_fft is redundant if you don't have FFTW installed
-option_names = ('boundary', 'interpolate_nan', 'normalize_kernel', 'ignore_edge_zeros', 'fft_type')
-options = list(itertools.product(BOUNDARY_OPTIONS, (True, False), (True, False), (True, False), ['numpy', 'scipy', 'fftw']))
+option_names = ('boundary', 'interpolate_nan', 'normalize_kernel', 'ignore_edge_zeros')
+options = list(itertools.product(BOUNDARY_OPTIONS, (True, False), (True, False), (True, False)))
 
 
 class TestConvolve1D(object):
 
     @pytest.mark.parametrize(option_names, options)
-    def test_unity_1_none(self, boundary, interpolate_nan, normalize_kernel, ignore_edge_zeros, fft_type):
+    def test_unity_1_none(self, boundary, interpolate_nan, normalize_kernel, ignore_edge_zeros):
         '''
         Test that a unit kernel with a single element returns the same array
         '''
 
-        if fft_type == 'fftw' and not HAS_FFTW:
-            pytest.skip('fftw3 is not installed')
-        elif fft_type == 'scipy' and not HAS_SCIPY:
-            pytest.skip('scipy is not installed')
-
         x = np.array([1., 2., 3.], dtype='float64')
 
         y = np.array([1.], dtype='float64')
@@ -64,23 +48,17 @@ class TestConvolve1D(object):
         z = convolve_fft(x, y, boundary=boundary,
                 interpolate_nan=interpolate_nan,
                 normalize_kernel=normalize_kernel,
-                ignore_edge_zeros=ignore_edge_zeros,
-                fft_type=fft_type)
+                ignore_edge_zeros=ignore_edge_zeros)
 
         assert_array_almost_equal_nulp(z, x, 10)
 
     @pytest.mark.parametrize(option_names, options)
-    def test_unity_3(self, boundary, interpolate_nan, normalize_kernel, ignore_edge_zeros, fft_type):
+    def test_unity_3(self, boundary, interpolate_nan, normalize_kernel, ignore_edge_zeros):
         '''
         Test that a unit kernel with three elements returns the same array
         (except when boundary is None).
         '''
 
-        if fft_type == 'fftw' and not HAS_FFTW:
-            pytest.skip('fftw3 is not installed')
-        elif fft_type == 'scipy' and not HAS_SCIPY:
-            pytest.skip('scipy is not installed')
-
         x = np.array([1., 2., 3.], dtype='float64')
 
         y = np.array([0., 1., 0.], dtype='float64')
@@ -88,23 +66,17 @@ class TestConvolve1D(object):
         z = convolve_fft(x, y, boundary=boundary,
                 interpolate_nan=interpolate_nan,
                 normalize_kernel=normalize_kernel,
-                ignore_edge_zeros=ignore_edge_zeros,
-                fft_type=fft_type)
+                ignore_edge_zeros=ignore_edge_zeros)
 
         assert_array_almost_equal_nulp(z, x, 10)
 
     @pytest.mark.parametrize(option_names, options)
-    def test_uniform_3(self, boundary, interpolate_nan, normalize_kernel, ignore_edge_zeros, fft_type):
+    def test_uniform_3(self, boundary, interpolate_nan, normalize_kernel, ignore_edge_zeros):
         '''
         Test that the different modes are producing the correct results using
         a uniform kernel with three elements
         '''
 
-        if fft_type == 'fftw' and not HAS_FFTW:
-            pytest.skip('fftw3 is not installed')
-        elif fft_type == 'scipy' and not HAS_SCIPY:
-            pytest.skip('scipy is not installed')
-
         x = np.array([1., 0., 3.], dtype='float64')
 
         y = np.array([1., 1., 1.], dtype='float64')
@@ -112,8 +84,7 @@ class TestConvolve1D(object):
         z = convolve_fft(x, y, boundary=boundary,
                 interpolate_nan=interpolate_nan,
                 normalize_kernel=normalize_kernel,
-                ignore_edge_zeros=ignore_edge_zeros,
-                fft_type=fft_type)
+                ignore_edge_zeros=ignore_edge_zeros)
 
         answer_dict = {
                 'sum': np.array([1., 4., 3.], dtype='float64'),
@@ -143,18 +114,13 @@ class TestConvolve1D(object):
         assert_array_almost_equal_nulp(z, answer_dict[answer_key], 10)
 
     @pytest.mark.parametrize(option_names, options)
-    def test_unity_3_withnan(self, boundary, interpolate_nan, normalize_kernel, ignore_edge_zeros, fft_type):
+    def test_unity_3_withnan(self, boundary, interpolate_nan, normalize_kernel, ignore_edge_zeros):
         '''
         Test that a unit kernel with three elements returns the same array
         (except when boundary is None). This version includes a NaN value in
         the original array.
         '''
 
-        if fft_type == 'fftw' and not HAS_FFTW:
-            pytest.skip('fftw3 is not installed')
-        elif fft_type == 'scipy' and not HAS_SCIPY:
-            pytest.skip('scipy is not installed')
-
         x = np.array([1., np.nan, 3.], dtype='float64')
 
         y = np.array([0., 1., 0.], dtype='float64')
@@ -162,32 +128,23 @@ class TestConvolve1D(object):
         z = convolve_fft(x, y, boundary=boundary,
                 interpolate_nan=interpolate_nan,
                 normalize_kernel=normalize_kernel,
-                ignore_edge_zeros=ignore_edge_zeros,
-                fft_type=fft_type)
+                ignore_edge_zeros=ignore_edge_zeros)
 
-        if fft_type == 'numpy':
-            # for whatever reason, numpy's fft has very limited precision, and
-            # the comparison fails unless you cast the float64 to a float16
-            # REMOVED because of numpy 1.4 incompatibility assert_array_almost_equal_nulp(np.asarray(z, dtype=np.float16), np.array([1.,0.,3.], dtype=np.float16), 10)
-            # ASSERT equality to better than 16 bit but worse than 32 bit precision
-            assert np.all(np.abs(z - np.array([1., 0., 3.])) < 1e-14)
-        else:
-            assert_array_almost_equal_nulp(z, np.array([1., 0., 3.], dtype='float64'), 10)
+        # for whatever reason, numpy's fft has very limited precision, and
+        # the comparison fails unless you cast the float64 to a float16
+        # REMOVED because of numpy 1.4 incompatibility assert_array_almost_equal_nulp(np.asarray(z, dtype=np.float16), np.array([1.,0.,3.], dtype=np.float16), 10)
+        # ASSERT equality to better than 16 bit but worse than 32 bit precision
+        assert np.all(np.abs(z - np.array([1., 0., 3.])) < 1e-14)
 
     @pytest.mark.parametrize(option_names, options)
     def test_unity_1_withnan(self, boundary, interpolate_nan, normalize_kernel,
-            ignore_edge_zeros, fft_type):
+            ignore_edge_zeros):
         '''
         Test that a unit kernel with three elements returns the same array
         (except when boundary is None). This version includes a NaN value in
         the original array.
         '''
 
-        if fft_type == 'fftw' and not HAS_FFTW:
-            pytest.skip('fftw3 is not installed')
-        elif fft_type == 'scipy' and not HAS_SCIPY:
-            pytest.skip('scipy is not installed')
-
         x = np.array([1., np.nan, 3.], dtype='float64')
 
         y = np.array([1.], dtype='float64')
@@ -195,31 +152,22 @@ class TestConvolve1D(object):
         z = convolve_fft(x, y, boundary=boundary,
                 interpolate_nan=interpolate_nan,
                 normalize_kernel=normalize_kernel,
-                ignore_edge_zeros=ignore_edge_zeros,
-                fft_type=fft_type)
+                ignore_edge_zeros=ignore_edge_zeros)
 
-        if fft_type == 'numpy':
-            # for whatever reason, numpy's fft has very limited precision, and
-            # the comparison fails unless you cast the float64 to a float16
-            # np1.4 incompatible assert_array_almost_equal_nulp(np.asarray(z, dtype=np.float16), np.array([1.,0.,3.], dtype=np.float16), 10)
-            assert np.all(np.abs(z - np.array([1., 0., 3.])) < 1e-14)
-        else:
-            assert_array_almost_equal_nulp(z, np.array([1., 0., 3.], dtype='float64'), 10)
+        # for whatever reason, numpy's fft has very limited precision, and
+        # the comparison fails unless you cast the float64 to a float16
+        # np1.4 incompatible assert_array_almost_equal_nulp(np.asarray(z, dtype=np.float16), np.array([1.,0.,3.], dtype=np.float16), 10)
+        assert np.all(np.abs(z - np.array([1., 0., 3.])) < 1e-14)
 
     @pytest.mark.parametrize(option_names, options)
     def test_uniform_3_withnan(self, boundary, interpolate_nan,
-            normalize_kernel, ignore_edge_zeros, fft_type):
+            normalize_kernel, ignore_edge_zeros):
         '''
         Test that the different modes are producing the correct results using
         a uniform kernel with three elements. This version includes a NaN
         value in the original array.
         '''
 
-        if fft_type == 'fftw' and not HAS_FFTW:
-            pytest.skip('fftw3 is not installed')
-        elif fft_type == 'scipy' and not HAS_SCIPY:
-            pytest.skip('scipy is not installed')
-
         x = np.array([1., np.nan, 3.], dtype='float64')
 
         y = np.array([1., 1., 1.], dtype='float64')
@@ -227,8 +175,7 @@ class TestConvolve1D(object):
         z = convolve_fft(x, y, boundary=boundary,
                 interpolate_nan=interpolate_nan,
                 normalize_kernel=normalize_kernel,
-                ignore_edge_zeros=ignore_edge_zeros,
-                fft_type=fft_type)
+                ignore_edge_zeros=ignore_edge_zeros)
 
         answer_dict = {
                 'sum': np.array([1., 4., 3.], dtype='float64'),
@@ -271,16 +218,11 @@ class TestConvolve2D(object):
 
     @pytest.mark.parametrize(option_names, options)
     def test_unity_1x1_none(self, boundary, interpolate_nan, normalize_kernel,
-            ignore_edge_zeros, fft_type):
+            ignore_edge_zeros):
         '''
         Test that a 1x1 unit kernel returns the same array
         '''
 
-        if fft_type == 'fftw' and not HAS_FFTW:
-            pytest.skip('fftw3 is not installed')
-        elif fft_type == 'scipy' and not HAS_SCIPY:
-            pytest.skip('scipy is not installed')
-
         x = np.array([[1., 2., 3.],
                       [4., 5., 6.],
                       [7., 8., 9.]], dtype='float64')
@@ -290,23 +232,17 @@ class TestConvolve2D(object):
         z = convolve_fft(x, y, boundary=boundary,
                 interpolate_nan=interpolate_nan,
                 normalize_kernel=normalize_kernel,
-                ignore_edge_zeros=ignore_edge_zeros,
-                fft_type=fft_type)
+                ignore_edge_zeros=ignore_edge_zeros)
 
         assert_array_almost_equal_nulp(z, x, 10)
 
     @pytest.mark.parametrize(option_names, options)
-    def test_unity_3x3(self, boundary, interpolate_nan, normalize_kernel, ignore_edge_zeros, fft_type):
+    def test_unity_3x3(self, boundary, interpolate_nan, normalize_kernel, ignore_edge_zeros):
         '''
         Test that a 3x3 unit kernel returns the same array (except when
         boundary is None).
         '''
 
-        if fft_type == 'fftw' and not HAS_FFTW:
-            pytest.skip('fftw3 is not installed')
-        elif fft_type == 'scipy' and not HAS_SCIPY:
-            pytest.skip('scipy is not installed')
-
         x = np.array([[1., 2., 3.],
                       [4., 5., 6.],
                       [7., 8., 9.]], dtype='float64')
@@ -318,24 +254,18 @@ class TestConvolve2D(object):
         z = convolve_fft(x, y, boundary=boundary,
                 interpolate_nan=interpolate_nan,
                 normalize_kernel=normalize_kernel,
-                ignore_edge_zeros=ignore_edge_zeros,
-                fft_type=fft_type)
+                ignore_edge_zeros=ignore_edge_zeros)
 
         assert_array_almost_equal_nulp(z, x, 10)
         #assert np.all( np.abs(z-x) < np.spacing(np.where(z>x,z,x))*2 )
 
     @pytest.mark.parametrize(option_names, options)
-    def test_uniform_3x3(self, boundary, interpolate_nan, normalize_kernel, ignore_edge_zeros, fft_type):
+    def test_uniform_3x3(self, boundary, interpolate_nan, normalize_kernel, ignore_edge_zeros):
         '''
         Test that the different modes are producing the correct results using
         a 3x3 uniform kernel.
         '''
 
-        if fft_type == 'fftw' and not HAS_FFTW:
-            pytest.skip('fftw3 is not installed')
-        elif fft_type == 'scipy' and not HAS_SCIPY:
-            pytest.skip('scipy is not installed')
-
         x = np.array([[0., 0., 3.],
                       [1., 0., 0.],
                       [0., 2., 0.]], dtype='float64')
@@ -347,8 +277,7 @@ class TestConvolve2D(object):
         z = convolve_fft(x, y, boundary=boundary,
                 interpolate_nan=interpolate_nan,
                 normalize_kernel=normalize_kernel,
-                ignore_edge_zeros=ignore_edge_zeros,
-                fft_type=fft_type)
+                ignore_edge_zeros=ignore_edge_zeros)
 
         w = np.array([[4., 6., 4.],
                       [6., 9., 6.],
@@ -381,21 +310,18 @@ class TestConvolve2D(object):
         print "z: ", z
         print "answer: ", a
         print "ratio: ", z / a
-        assert np.all(np.abs(z - a) < np.spacing(np.where(z > a, z, a)) * 10)
+        # for reasons unknown, the Windows FFT returns an answer for the [0,0]
+        # component that is EXACTLY 10*np.spacing
+        assert np.all(np.abs(z - a) <= np.spacing(np.where(z > a, z, a)) * 10)
 
     @pytest.mark.parametrize(option_names, options)
-    def test_unity_3x3_withnan(self, boundary, interpolate_nan, normalize_kernel, ignore_edge_zeros, fft_type):
+    def test_unity_3x3_withnan(self, boundary, interpolate_nan, normalize_kernel, ignore_edge_zeros):
         '''
         Test that a 3x3 unit kernel returns the same array (except when
         boundary is None). This version includes a NaN value in the original
         array.
         '''
 
-        if fft_type == 'fftw' and not HAS_FFTW:
-            pytest.skip('fftw3 is not installed')
-        elif fft_type == 'scipy' and not HAS_SCIPY:
-            pytest.skip('scipy is not installed')
-
         x = np.array([[1., 2., 3.],
                       [4., np.nan, 6.],
                       [7., 8., 9.]], dtype='float64')
@@ -408,32 +334,24 @@ class TestConvolve2D(object):
                 interpolate_nan=interpolate_nan,
                 normalize_kernel=normalize_kernel,
                 ignore_edge_zeros=ignore_edge_zeros,
-                fft_type=fft_type)
+                )
 
         a = x
         a[1, 1] = 0
 
-        if fft_type == 'numpy':
-            # for whatever reason, numpy's fft has very limited precision, and
-            # the comparison fails unless you cast the float64 to a float16
-            # np1.4 incompatible assert_array_almost_equal_nulp(np.asarray(z, dtype=np.float16), np.asarray(a, dtype=np.float16), 10)
-            assert np.all(np.abs(z - a) < 1e-14)
-        else:
-            assert_array_almost_equal_nulp(z, a, 10)
+        # for whatever reason, numpy's fft has very limited precision, and
+        # the comparison fails unless you cast the float64 to a float16
+        # np1.4 incompatible assert_array_almost_equal_nulp(np.asarray(z, dtype=np.float16), np.asarray(a, dtype=np.float16), 10)
+        assert np.all(np.abs(z - a) < 1e-14)
 
     @pytest.mark.parametrize(option_names, options)
-    def test_uniform_3x3_withnan(self, boundary, interpolate_nan, normalize_kernel, ignore_edge_zeros, fft_type):
+    def test_uniform_3x3_withnan(self, boundary, interpolate_nan, normalize_kernel, ignore_edge_zeros):
         '''
         Test that the different modes are producing the correct results using
         a 3x3 uniform kernel. This version includes a NaN value in the
         original array.
         '''
 
-        if fft_type == 'fftw' and not HAS_FFTW:
-            pytest.skip('fftw3 is not installed')
-        elif fft_type == 'scipy' and not HAS_SCIPY:
-            pytest.skip('scipy is not installed')
-
         x = np.array([[0., 0., 3.],
                       [1., np.nan, 0.],
                       [0., 2., 0.]], dtype='float64')
@@ -446,7 +364,7 @@ class TestConvolve2D(object):
                 interpolate_nan=interpolate_nan,
                 normalize_kernel=normalize_kernel,
                 ignore_edge_zeros=ignore_edge_zeros,
-                fft_type=fft_type)
+                )
 
         w_n = np.array([[3., 5., 3.],
                         [5., 8., 5.],
@@ -491,4 +409,6 @@ class TestConvolve2D(object):
         print "z: ", z
         print "answer: ", a
         print "ratio: ", z / a
-        assert np.all(np.abs(z - a) < np.spacing(np.where(z > a, z, a)) * 10)
+        # for reasons unknown, the Windows FFT returns an answer for the [0,0]
+        # component that is EXACTLY 10*np.spacing
+        assert np.all(np.abs(z - a) <= np.spacing(np.where(z > a, z, a)) * 10)
