@@ -11,8 +11,8 @@ but can be used independently.
 =======================
 
 This directive will produce an "autosummary"-style table for public
-attributesof a specified module. See the `sphinx.ext.autosummary`
-extensionfor details on this process. The main difference from the
+attributes of a specified module. See the `sphinx.ext.autosummary`
+extension for details on this process. The main difference from the
 `autosummary` directive is that `autosummary` requires manually inputting
 all attributes that appear in the table, while this captures the entries
 automatically.
@@ -34,6 +34,11 @@ options:
         functions. This cannot be used at the same time with
         ``:classes-only:`` .
 
+    * ``:skip: obj1, [obj2, obj3, ...]``
+        If present, specifies that the listed objects should be skipped
+        and not have their documentation generated, nor be includded in
+        the summary table.
+
 
 ===========================
 `automod-diagram` directive
@@ -60,6 +65,17 @@ from ...utils.misc import find_mod_objs
 from .astropyautosummary import AstropyAutosummary
 
 
+def _str_list_converter(argument):
+    """
+    A directive option conversion function that converts the option into a list
+    of strings. Used for 'skip' option.
+    """
+    if argument is None:
+        return []
+    else:
+        return [s.strip() for s in argument.split(',')]
+
+
 class Automodsumm(AstropyAutosummary):
     required_arguments = 1
     optional_arguments = 0
@@ -68,6 +84,7 @@ class Automodsumm(AstropyAutosummary):
     option_spec = dict(Autosummary.option_spec)
     option_spec['functions-only'] = flag
     option_spec['classes-only'] = flag
+    option_spec['skip'] = _str_list_converter
 
     def run(self):
         from inspect import isclass, isfunction
@@ -86,21 +103,35 @@ class Automodsumm(AstropyAutosummary):
             # Be sure to respect functions-only and classes-only.
             funconly = 'functions-only' in self.options
             clsonly = 'classes-only' in self.options
+
+            skipmap = {}
+            if 'skip' in self.options:
+                skipnames = set(self.options['skip'])
+                for lnm, fqnm in zip(localnames, fqns):
+                    if lnm in skipnames:
+                        skipnames.remove(lnm)
+                        skipmap[fqnm] = lnm
+                if len(skipnames) > 0:
+                    self.warn('Tried to skip objects {objs} in module {mod}, '
+                              'but they were not present.  Ignoring.'.format(
+                              objs=skipnames, mod=self.arguments[0]))
+
             if funconly and not clsonly:
                 cont = []
                 for nm, obj in zip(fqns, objs):
-                    if isfunction(obj):
+                    if nm not in skipmap and isfunction(obj):
                         cont.append('~' + nm)
             elif clsonly:
                 cont = []
                 for nm, obj in zip(fqns, objs):
-                    if isclass(obj):
+                    if nm not in skipmap and isclass(obj):
                         cont.append('~' + nm)
             else:
                 if clsonly and funconly:
                     self.warning('functions-only and classes-only both '
                                  'defined. Skipping.')
-                cont = ['~' + objname for objname in fqns]
+                cont = ['~' + nm for nm in fqns if nm not in skipmap]
+
             self.content = cont
 
             #can't use super because Sphinx/docutils has trouble
@@ -291,7 +322,8 @@ def generate_automodsumm_docs(lines, srcfn, suffix='.rst', warn=None,
     # Create our own templating environment - here we use Astropy's
     # templates rather than the default autosummary templates, in order to
     # allow docstrings to be shown for methods.
-    template_dirs = [os.path.join(base_path, '_templates')]
+    template_dirs = [os.path.join(os.path.dirname(__file__), 'templates'),
+                     os.path.join(base_path, '_templates')]
     if builder is not None:
         # allow the user to override the templates
         template_loader = BuiltinTemplateLoader()
@@ -375,7 +407,8 @@ def generate_automodsumm_docs(lines, srcfn, suffix='.rst', warn=None,
                           if x in include_public or not x.startswith('_')]
                 return public, items
 
-            def get_members_class(obj, typ, include_public=[], include_base=False):
+            def get_members_class(obj, typ, include_public=[],
+                                  include_base=False):
                 """
                 typ = None -> all
                 include_base -> include attrs that are from a base class
@@ -420,6 +453,8 @@ def generate_automodsumm_docs(lines, srcfn, suffix='.rst', warn=None,
                                  get_members_class(obj, 'method', ['__init__'])
                 ns['attributes'], ns['all_attributes'] = \
                                  get_members_class(obj, 'attribute')
+                ns['methods'].sort()
+                ns['attributes'].sort()
 
             parts = name.split('.')
             if doc.objtype in ('method', 'attribute'):
@@ -448,7 +483,10 @@ def generate_automodsumm_docs(lines, srcfn, suffix='.rst', warn=None,
             # astropy.config.logging_helper (which is at
             # astropy/config/logging_helper.py) then the reference file is set
             # to ../config/references.txt
-            mod_name_dir = mod_name.replace('.', '/').split('/', 1)[1]
+            if '.' in mod_name:
+                mod_name_dir = mod_name.replace('.', '/').split('/', 1)[1]
+            else:
+                mod_name_dir = mod_name
             if not os.path.isdir(os.path.join(path, '..', mod_name_dir)) \
                and os.path.isdir(os.path.join(path, '..', mod_name_dir.rsplit('/', 1)[0])):
                 mod_name_dir = mod_name_dir.rsplit('/', 1)[0]
