@@ -1,51 +1,114 @@
-import pytest
+# Licensed under a 3-clause BSD style license - see LICENSE.rst
+from distutils import version
 import numpy as np
 
-from .. import Table, Column
+from ...tests.helper import pytest
+from ... import table
+
+numpy_lt_1p5 = version.LooseVersion(np.__version__) < version.LooseVersion('1.5')
+
+# Dummy init of Table, DATA for pyflakes and to be sure test fixture is working
+Table = None
+Column = None
 
 
+class MaskedTable(table.Table):
+    def __init__(self, *args, **kwargs):
+        kwargs['masked'] = True
+        table.Table.__init__(self, *args, **kwargs)
+
+
+# Fixture to run all the Column tests for both an unmasked (ndarray)
+# and masked (MaskedArray) column.
+@pytest.fixture(params=[False] if numpy_lt_1p5 else [False, True])
+def set_global_Table(request):
+    global Table, Column
+
+    Table = MaskedTable if request.param else table.Table
+    Column = table.MaskedColumn if request.param else table.Column
+
+
+class SetupData(object):
+    @property
+    def a(self):
+        if Column is not None:
+            if not hasattr(self, '_a'):
+                self._a = Column(name='a', data=[1, 2, 3], meta={'aa': np.arange(5)})
+            return self._a
+
+    @property
+    def b(self):
+        if Column is not None:
+            if not hasattr(self, '_b'):
+                self._b = Column(name='b', data=[4, 5, 6])
+            return self._b
+
+    @property
+    def c(self):
+        if Column is not None:
+            if not hasattr(self, '_c'):
+                self._c = Column(name='c', data=[7, 8, 9])
+            return self._c
+
+    @property
+    def d(self):
+        if Column is not None:
+            if not hasattr(self, '_d'):
+                self._d = Column(name='d', data=[7, 8, 7])
+            return self._d
+
+    @property
+    def t(self):
+        if Table is not None:
+            if not hasattr(self, '_t'):
+                self._t = Table([self.a, self.b])
+            return self._t
+
+
+@pytest.mark.usefixtures('set_global_Table')
 class TestEmptyData():
 
     def test_1(self):
         t = Table()
-        t.add_column(Column('a', dtype=int, length=100))
+        t.add_column(Column(name='a', dtype=int, length=100))
         assert len(t['a']) == 100
 
     def test_2(self):
         t = Table()
-        t.add_column(Column('a', dtype=int, shape=(3, ), length=100))
+        t.add_column(Column(name='a', dtype=int, shape=(3, ), length=100))
         assert len(t['a']) == 100
 
     def test_3(self):
         t = Table()  # length is not given
-        t.add_column(Column('a', dtype=int))
+        t.add_column(Column(name='a', dtype=int))
         assert len(t['a']) == 0
 
     def test_4(self):
         t = Table()  # length is not given
-        t.add_column(Column('a', dtype=int, shape=(3, 4)))
+        t.add_column(Column(name='a', dtype=int, shape=(3, 4)))
         assert len(t['a']) == 0
 
     def test_5(self):
         t = Table()
-        t.add_column(Column('a'))  # dtype is not specified
+        t.add_column(Column(name='a'))  # dtype is not specified
         assert len(t['a']) == 0
 
 
+@pytest.mark.usefixtures('set_global_Table')
 class TestNewFromColumns():
 
     def test_simple(self):
-        cols = [Column('a', [1, 2, 3]),
-                Column('b', [4, 5, 6], dtype=np.float32)]
+        cols = [Column(name='a', data=[1, 2, 3]),
+                Column(name='b', data=[4, 5, 6], dtype=np.float32)]
         t = Table(cols)
         assert np.all(t['a'].data == np.array([1, 2, 3]))
         assert np.all(t['b'].data == np.array([4, 5, 6], dtype=np.float32))
         assert type(t['b'][1]) == np.float32
 
     def test_from_np_array(self):
-        cols = [Column('a', np.array([1, 2, 3], dtype=np.int64),
+        cols = [Column(name='a', data=np.array([1, 2, 3], dtype=np.int64),
                        dtype=np.float64),
-                Column('b', np.array([4, 5, 6], dtype=np.float32))]
+                Column(name='b', data=np.array([4, 5, 6], dtype=np.float32))]
         t = Table(cols)
         assert np.all(t['a'] == np.array([1, 2, 3], dtype=np.float64))
         assert np.all(t['b'] == np.array([4, 5, 6], dtype=np.float32))
@@ -53,12 +116,20 @@ class TestNewFromColumns():
         assert type(t['b'][1]) == np.float32
 
     def test_size_mismatch(self):
-        cols = [Column('a', [1, 2, 3]),
-                Column('b', [4, 5, 6, 7])]
+        cols = [Column(name='a', data=[1, 2, 3]),
+                Column(name='b', data=[4, 5, 6, 7])]
         with pytest.raises(ValueError):
             Table(cols)
 
+    def test_name_none(self):
+        """Column with name=None can init a table IFF names are supplied"""
+        c = Column(data=[1, 2])
+        Table([c], names=('c',))
+        with pytest.raises(TypeError):
+            Table([c])
 
+
+@pytest.mark.usefixtures('set_global_Table')
 class TestReverse():
 
     def test_reverse(self):
@@ -81,6 +152,7 @@ class TestReverse():
         assert np.all(t2['col1'] == np.array(['a', 'b', 'cc']))
 
 
+@pytest.mark.usefixtures('set_global_Table')
 class TestColumnAccess():
 
     def test_1(self):
@@ -90,16 +162,14 @@ class TestColumnAccess():
 
     def test_2(self):
         t = Table()
-        t.add_column(Column('a', [1, 2, 3]))
+        t.add_column(Column(name='a', data=[1, 2, 3]))
         assert np.all(t['a'] == np.array([1, 2, 3]))
         with pytest.raises(KeyError):
             t['b']  # column does not exist
 
 
-class TestAddLength():
-    def setup_method(self, method):
-        self.a = Column('a', [1, 2, 3])
-        self.b = Column('b', [4, 5, 6])
+@pytest.mark.usefixtures('set_global_Table')
+class TestAddLength(SetupData):
 
     def test_right_length(self):
         t = Table([self.a])
@@ -108,19 +178,16 @@ class TestAddLength():
     def test_too_long(self):
         t = Table([self.a])
         with pytest.raises(ValueError):
-            t.add_column(Column('b', [4, 5, 6, 7]))  # data too long
+            t.add_column(Column(name='b', data=[4, 5, 6, 7]))  # data too long
 
     def test_too_short(self):
         t = Table([self.a])
         with pytest.raises(ValueError):
-            t.add_column(Column('b', [4, 5]))  # data too short
+            t.add_column(Column(name='b', data=[4, 5]))  # data too short
 
 
-class TestAddPosition():
-    def setup_method(self, method):
-        self.a = Column('a', [1, 2, 3])
-        self.b = Column('b', [4, 5, 6])
-        self.c = Column('c', [7, 8, 9])
+@pytest.mark.usefixtures('set_global_Table')
+class TestAddPosition(SetupData):
 
     def test_1(self):
         t = Table()
@@ -171,13 +238,8 @@ class TestAddPosition():
         assert t.columns.keys() == ['c', 'a', 'b']
 
 
-class TestInitFromTable():
-
-    def setup_method(self, method):
-        self.a = Column('a', [1, 2, 3], meta={'aa': np.arange(5)})
-        self.b = Column('b', [4, 5, 6])
-        self.c = Column('c', [7, 8, 9])
-        self.t = Table([self.a, self.b])
+@pytest.mark.usefixtures('set_global_Table')
+class TestInitFromTable(SetupData):
 
     def test_from_table_cols(self):
         """Ensure that using cols from an existing table gives
@@ -211,13 +273,8 @@ class TestInitFromTable():
             assert t.columns['a'].meta['aa'][3] == 3
 
 
-class TestAddColumns():
-
-    def setup_method(self, method):
-        self.a = Column('a', [1, 2, 3])
-        self.b = Column('b', [4, 5, 6])
-        self.c = Column('c', [7, 8, 9])
-        self.d = Column('d', [7, 8, 7])
+@pytest.mark.usefixtures('set_global_Table')
+class TestAddColumns(SetupData):
 
     def test_add_columns1(self):
         t = Table()
@@ -249,7 +306,7 @@ class TestAddColumns():
         t = Table()
         t.add_column(self.a)
         with pytest.raises(ValueError):
-            t.add_column(Column('a', [0, 1, 2]))
+            t.add_column(Column(name='a', data=[0, 1, 2]))
         t.add_column(self.b)
         t.add_column(self.c)
         assert t.colnames == ['a', 'b', 'c']
@@ -257,18 +314,43 @@ class TestAddColumns():
     def test_add_duplicate_columns(self):
         t = Table([self.a, self.b, self.c])
         with pytest.raises(ValueError):
-            t.add_columns([Column('a', [0, 1, 2]), Column('b', [0, 1, 2])])
+            t.add_columns([Column(name='a', data=[0, 1, 2]), Column(name='b', data=[0, 1, 2])])
         t.add_column(self.d)
         assert t.colnames == ['a', 'b', 'c', 'd']
 
 
-class TestAddRow():
+@pytest.mark.usefixtures('set_global_Table')
+class TestAddRow(SetupData):
 
-    def setup_method(self, method):
-        self.a = Column('a', [1, 2, 3])
-        self.b = Column('b', [4.0, 5.1, 6.2])
-        self.c = Column('c', ['7', '8', '9'])
-        self.t = Table([self.a, self.b, self.c])
+    @property
+    def b(self):
+        if Column is not None:
+            if not hasattr(self, '_b'):
+                self._b = Column(name='b', data=[4.0, 5.1, 6.2])
+            return self._b
+
+    @property
+    def c(self):
+        if Column is not None:
+            if not hasattr(self, '_c'):
+                self._c = Column(name='c', data=['7', '8', '9'])
+            return self._c
+
+    @property
+    def t(self):
+        if Table is not None:
+            if not hasattr(self, '_t'):
+                self._t = Table([self.a, self.b, self.c])
+            return self._t
+
+    def test_add_table_row(self):
+        t = self.t
+        t2 = Table([self.a, self.b, self.c])
+        t.add_row(t2[0])
+        assert len(t) == 4
+        assert np.all(t['a'] == np.array([1, 2, 3, 1]))
+        assert np.allclose(t['b'], np.array([4.0, 5.1, 6.2, 4.0]))
+        assert np.all(t['c'] == np.array(['7', '8', '9', '7']))
 
     def test_add_with_tuple(self):
         t = self.t
@@ -292,15 +374,23 @@ class TestAddRow():
         assert len(t) == 4
         assert np.all(t['a'] == np.array([1, 2, 3, 4]))
         assert np.allclose(t['b'], np.array([4.0, 5.1, 6.2, 7.2]))
-        assert np.all(t['c'] == np.array(['7', '8', '9', '']))
+        if t.masked:
+            assert np.all(t['c'] == np.array(['7', '8', '9', '7']))
+        else:
+            assert np.all(t['c'] == np.array(['7', '8', '9', '']))
 
     def test_add_with_none(self):
         t = self.t
         t.add_row()
         assert len(t) == 4
-        assert np.all(t['a'].data == np.array([1, 2, 3, 0]))
-        assert np.allclose(t['b'], np.array([4.0, 5.1, 6.2, 0.0]))
-        assert np.all(t['c'].data == np.array(['7', '8', '9', '']))
+        if t.masked:
+            assert np.all(t['a'].data == np.array([1, 2, 3, 1]))
+            assert np.allclose(t['b'], np.array([4.0, 5.1, 6.2, 4.0]))
+            assert np.all(t['c'].data == np.array(['7', '8', '9', '7']))
+        else:
+            assert np.all(t['a'].data == np.array([1, 2, 3, 0]))
+            assert np.allclose(t['b'], np.array([4.0, 5.1, 6.2, 0.0]))
+            assert np.all(t['c'].data == np.array(['7', '8', '9', '']))
 
     def test_add_missing_column(self):
         t = self.t
@@ -323,17 +413,13 @@ class TestAddRow():
                          (3, 4, 5)],
                         dtype='i4')
         t = Table(data, copy=False)
-        with pytest.raises(ValueError):
-            t.add_row([6, 7, 8])
+        if not t.masked:
+            with pytest.raises(ValueError):
+                t.add_row([6, 7, 8])
 
 
-class TestTableColumn():
-
-    def setup_method(self, method):
-        self.a = Column('a', [1, 2, 3], meta={'aa': np.arange(5)})
-        self.b = Column('b', [4, 5, 6])
-        self.c = Column('c', [7, 8, 9])
-        self.t = Table([self.a, self.b])
+@pytest.mark.usefixtures('set_global_Table')
+class TestTableColumn(SetupData):
 
     def test_column_view(self):
         t = self.t
@@ -342,20 +428,18 @@ class TestTableColumn():
         assert t._data['a'][2] == 10
 
 
-class TestArrayColumns():
-
-    def setup_method(self, method):
-        self.a = Column('a', [1, 2, 3])
+@pytest.mark.usefixtures('set_global_Table')
+class TestArrayColumns(SetupData):
 
     def test_1d(self):
-        b = Column('b', dtype=int, shape=(2, ), length=3)
+        b = Column(name='b', dtype=int, shape=(2, ), length=3)
         t = Table([self.a])
         t.add_column(b)
         assert t['b'].shape == (3, 2)
         assert t['b'][0].shape == (2, )
 
     def test_2d(self):
-        b = Column('b', dtype=int, shape=(2, 4), length=3)
+        b = Column(name='b', dtype=int, shape=(2, 4), length=3)
         t = Table([self.a])
         t.add_column(b)
         assert t['b'].shape == (3, 2, 4)
@@ -363,20 +447,28 @@ class TestArrayColumns():
 
     def test_3d(self):
         t = Table([self.a])
-        b = Column('b', dtype=int, shape=(2, 4, 6), length=3)
+        b = Column(name='b', dtype=int, shape=(2, 4, 6), length=3)
         t.add_column(b)
         assert t['b'].shape == (3, 2, 4, 6)
         assert t['b'][0].shape == (2, 4, 6)
 
 
-class TestRemove():
+@pytest.mark.usefixtures('set_global_Table')
+class TestRemove(SetupData):
 
-    def setup_method(self, method):
-        self.a = Column('a', [1, 2, 3])
-        self.b = Column('b', [4, 5, 6])
-        self.c = Column('c', [7, 8, 9])
-        self.t = Table([self.a])
-        self.t2 = Table([self.a, self.b, self.c])
+    @property
+    def t(self):
+        if Table is not None:
+            if not hasattr(self, '_t'):
+                self._t = Table([self.a])
+            return self._t
+
+    @property
+    def t2(self):
+        if Table is not None:
+            if not hasattr(self, '_t2'):
+                self._t2 = Table([self.a, self.b, self.c])
+            return self._t2
 
     def test_1(self):
         self.t.remove_columns('a')
@@ -408,11 +500,8 @@ class TestRemove():
             del self.t['d']
 
 
-class TestKeep():
-
-    def setup_method(self, method):
-        self.a = Column('a', [1, 2, 3])
-        self.b = Column('b', [4, 5, 6])
+@pytest.mark.usefixtures('set_global_Table')
+class TestKeep(SetupData):
 
     def test_1(self):
         t = Table([self.a, self.b])
@@ -428,11 +517,8 @@ class TestKeep():
         assert np.all(t['b'] == np.array([4, 5, 6]))
 
 
-class TestRename():
-
-    def setup_method(self, method):
-        self.a = Column('a', [1, 2, 3])
-        self.b = Column('b', [4, 5, 6])
+@pytest.mark.usefixtures('set_global_Table')
+class TestRename(SetupData):
 
     def test_1(self):
         t = Table([self.a])
@@ -460,12 +546,13 @@ class TestRename():
         assert np.all(t['a'] == np.array([4, 5, 6]))
 
 
+@pytest.mark.usefixtures('set_global_Table')
 class TestSort():
 
     def test_single(self):
         t = Table()
-        t.add_column(Column('a', [2, 1, 3]))
-        t.add_column(Column('b', [6, 5, 4]))
+        t.add_column(Column(name='a', data=[2, 1, 3]))
+        t.add_column(Column(name='b', data=[6, 5, 4]))
         assert np.all(t['a'] == np.array([2, 1, 3]))
         assert np.all(t['b'] == np.array([6, 5, 4]))
         t.sort('a')
@@ -477,8 +564,8 @@ class TestSort():
 
     def test_multiple(self):
         t = Table()
-        t.add_column(Column('a', [2, 1, 3, 2, 3, 1]))
-        t.add_column(Column('b', [6, 5, 4, 3, 5, 4]))
+        t.add_column(Column(name='a', data=[2, 1, 3, 2, 3, 1]))
+        t.add_column(Column(name='b', data=[6, 5, 4, 3, 5, 4]))
         assert np.all(t['a'] == np.array([2, 1, 3, 2, 3, 1]))
         assert np.all(t['b'] == np.array([6, 5, 4, 3, 5, 4]))
         t.sort(['a', 'b'])
@@ -489,6 +576,7 @@ class TestSort():
         assert np.all(t['b'] == np.array([3, 4, 4, 5, 5, 6]))
 
 
+@pytest.mark.usefixtures('set_global_Table')
 class TestIterator():
 
     def test_iterator(self):
@@ -496,10 +584,15 @@ class TestIterator():
                       (3, 6),
                       (4, 5)], dtype=[('a', 'i4'), ('b', 'i4')])
         t = Table(d)
-        for row, np_row in zip(t, d):
-            assert np.all(row == np_row)
+        if t.masked:
+            with pytest.raises(ValueError):
+                t[0] == d[0]
+        else:
+            for row, np_row in zip(t, d):
+                assert np.all(row == np_row)
 
 
+@pytest.mark.usefixtures('set_global_Table')
 class TestSetMeta():
 
     def test_set_meta(self):
@@ -511,19 +604,22 @@ class TestSetMeta():
         assert list(d.meta.keys()) == ['a', 'b', 'c', 'd']
 
 
+@pytest.mark.usefixtures('set_global_Table')
 class TestConvertNumpyArray():
 
     def test_convert_numpy_array(self):
         d = Table([[1, 2], [3, 4]], names=('a', 'b'))
 
         np_data = np.array(d)
-        assert np.all(np_data == d._data)
+        if Table is not MaskedTable:
+            assert np.all(np_data == d._data)
         assert not np_data is d._data
         assert d.colnames == list(np_data.dtype.names)
 
         np_data = np.array(d, copy=False)
-        assert np.all(np_data == d._data)
-        assert np_data is d._data
+        if Table is not MaskedTable:
+            assert np.all(np_data == d._data)
+            assert np_data is d._data
         assert d.colnames == list(np_data.dtype.names)
 
         with pytest.raises(ValueError):
