@@ -14,11 +14,19 @@ import subprocess
 import shutil
 import tempfile
 
+try:
+    # Import pkg_resources to prevent it from issuing warnings upon being
+    # imported from within py.test.  See
+    # https://github.com/astropy/astropy/pull/537 for a detailed explanation.
+    import pkg_resources
+except ImportError:
+    pass
+
 from distutils.core import Command
 
 from .. import test
 
-if os.environ.get('ASTROPY_USE_SYSTEM_PYTEST'):
+if os.environ.get('ASTROPY_USE_SYSTEM_PYTEST') or '_pytest' in sys.modules:
     import pytest
 
 else:
@@ -56,7 +64,7 @@ class TestRunner(object):
 
     def run_tests(self, package=None, test_path=None, args=None, plugins=None,
                   verbose=False, pastebin=None, remote_data=False, pep8=False,
-                  pdb=False, coverage=False):
+                  pdb=False, coverage=False, open_files=False):
         """
         The docstring for this method lives in astropy/__init__.py:test
         """
@@ -115,7 +123,7 @@ class TestRunner(object):
                     'Coverage reporting requires pytest-cov plugin: '
                     'http://pypi.python.org/pypi/pytest-cov')
             else:
-                # Don't use get_data_filename here, because it
+                # Don't use get_pkg_data_filename here, because it
                 # requires importing astropy.config and thus screwing
                 # up coverage results for those packages.
                 coveragerc = os.path.join(
@@ -139,23 +147,21 @@ class TestRunner(object):
                     ' --cov-report html --cov astropy'
                     ' --cov-config {0}'.format(tmp.name))
 
-        # check if we're inside the distutils test command, which sets the
-        # _ASTROPY_TEST_ builtin
-        try:
-            _ASTROPY_TEST_
-            insidetestcmd = True
-        except NameError:
-            insidetestcmd = False
+        # check for opened files after each test
+        if open_files:
+            try:
+                subproc = subprocess.Popen(
+                    ['lsof -F0 -n -p {0}'.format(os.getpid())],
+                    shell=True, stdout=subprocess.PIPE)
+                output = subproc.communicate()[0].strip()
+            except subprocess.CalledProcessError:
+                raise SystemError(
+                    "open file detection requested, but could not "
+                    "successfully run the 'lsof' command")
 
-        #set up environment variables to fake the config and cache systems,
-        #unless this has already been done by the distutils test command
-        if not insidetestcmd:
-            oldconfigdir = os.environ.get('XDG_CONFIG_HOME')
-            oldcachedir = os.environ.get('XDG_CACHE_HOME')
-            os.environ['XDG_CONFIG_HOME'] = tempfile.mkdtemp('astropy_config')
-            os.environ['XDG_CACHE_HOME'] = tempfile.mkdtemp('astropy_cache')
-            os.mkdir(os.path.join(os.environ['XDG_CONFIG_HOME'], 'astropy'))
-            os.mkdir(os.path.join(os.environ['XDG_CACHE_HOME'], 'astropy'))
+            all_args += ' --open-files'
+
+            print("Checking for unclosed files")
 
         try:
             all_args = shlex.split(
@@ -168,20 +174,8 @@ class TestRunner(object):
                     tmp.close()
                 os.remove(tmp.name)
 
-            if not insidetestcmd:
-                #wipe the config/cache tmpdirs and restore the envars
-                shutil.rmtree(os.environ['XDG_CONFIG_HOME'])
-                shutil.rmtree(os.environ['XDG_CACHE_HOME'])
-                if oldconfigdir is None:
-                    del os.environ['XDG_CONFIG_HOME']
-                else:
-                    os.environ['XDG_CONFIG_HOME'] = oldconfigdir
-                if oldcachedir is None:
-                    del os.environ['XDG_CACHE_HOME']
-                else:
-                    os.environ['XDG_CACHE_HOME'] = oldcachedir
-
         return result
+
     run_tests.__doc__ = test.__doc__
 
 
@@ -209,7 +203,8 @@ class astropy_test(Command, object):
         ('pdb', 'd', 'Turn on PDB post-mortem analysis for failing tests. '
          'Same as specifying `--pdb` in `args`.'),
         ('coverage', 'c', 'Create a coverage report. Requires the pytest-cov '
-         'plugin is installed')
+         'plugin is installed'),
+        ('open-files', 'o', 'Fail if any tests leave files open')
     ]
 
     package_name = None
@@ -225,6 +220,7 @@ class astropy_test(Command, object):
         self.pep8 = False
         self.pdb = False
         self.coverage = False
+        self.open_files = False
 
     def finalize_options(self):
         # Normally we would validate the options here, but that's handled in
@@ -237,46 +233,68 @@ class astropy_test(Command, object):
         build_cmd = self.get_finalized_command('build')
         new_path = os.path.abspath(build_cmd.build_lib)
 
-        # Run the tests in a subprocess--this is necessary since new extension
-        # modules may have appeared, and this is the easiest way to set up a
-        # new environment
-
-        # We need to set a flag in the child's environment so that
-        # unnecessary code is not imported before py.test can start
-        # up, otherwise the coverage results will be artifically low.
-        if sys.version_info[0] >= 3:
-            set_flag = "import builtins; builtins._ASTROPY_TEST_ = True"
-        else:
-            set_flag = "import __builtin__; __builtin__._ASTROPY_TEST_ = True"
-
-        cmd = ('{0}; import {1.package_name}, sys; sys.exit('
-               '{1.package_name}.test({1.package!r}, {1.test_path!r}, '
-               '{1.args!r}, {1.plugins!r}, {1.verbose_results!r}, '
-               '{1.pastebin!r}, {1.remote_data!r}, {1.pep8!r}, {1.pdb!r}, '
-               '{1.coverage!r}))')
-        cmd = cmd.format(set_flag, self)
-
-        #override the config locations to not make a new directory nor use
-        #existing cache or config
-        os.environ['XDG_CONFIG_HOME'] = tempfile.mkdtemp('astropy_config')
-        os.environ['XDG_CACHE_HOME'] = tempfile.mkdtemp('astropy_cache')
-        os.mkdir(os.path.join(os.environ['XDG_CONFIG_HOME'], 'astropy'))
-        os.mkdir(os.path.join(os.environ['XDG_CACHE_HOME'], 'astropy'))
+        # Copy the build to a temporary directory for the purposes of testing
+        # - this avoids creating pyc and __pycache__ directories inside the
+        # build directory
+        tmp_dir = tempfile.mkdtemp(prefix='astropy-test-')
+        testing_path = os.path.join(tmp_dir, os.path.basename(new_path))
+        shutil.copytree(new_path, testing_path)
 
         try:
-            retcode = subprocess.call([sys.executable, '-c', cmd],
-                                      cwd=new_path, close_fds=False)
-        finally:
-            # kill the temporary dirs
-            shutil.rmtree(os.environ['XDG_CONFIG_HOME'])
-            shutil.rmtree(os.environ['XDG_CACHE_HOME'])
 
-        if self.coverage and retcode == 0:
-            # Copy the htmlcov from build/lib.../htmlcov to a more
-            # obvious place
-            if os.path.exists('htmlcov'):
-                shutil.rmtree('htmlcov')
-            shutil.copytree(os.path.join(new_path, 'htmlcov'), 'htmlcov')
+            # Run the tests in a subprocess--this is necessary since new extension
+            # modules may have appeared, and this is the easiest way to set up a
+            # new environment
+
+            # We need to set a flag in the child's environment so that
+            # unnecessary code is not imported before py.test can start
+            # up, otherwise the coverage results will be artifically low.
+            if sys.version_info[0] >= 3:
+                set_flag = "import builtins; builtins._ASTROPY_TEST_ = True"
+            else:
+                set_flag = "import __builtin__; __builtin__._ASTROPY_TEST_ = True"
+
+            cmd = ('{0}; import {1.package_name}, sys; sys.exit('
+                   '{1.package_name}.test('
+                   'package={1.package!r}, '
+                   'test_path={1.test_path!r}, '
+                   'args={1.args!r}, '
+                   'plugins={1.plugins!r}, '
+                   'verbose={1.verbose_results!r}, '
+                   'pastebin={1.pastebin!r}, '
+                   'remote_data={1.remote_data!r}, '
+                   'pep8={1.pep8!r}, '
+                   'pdb={1.pdb!r}, '
+                   'coverage={1.coverage!r}, '
+                   'open_files={1.open_files!r}))')
+            cmd = cmd.format(set_flag, self)
+
+            #override the config locations to not make a new directory nor use
+            #existing cache or config
+            os.environ['XDG_CONFIG_HOME'] = tempfile.mkdtemp('astropy_config')
+            os.environ['XDG_CACHE_HOME'] = tempfile.mkdtemp('astropy_cache')
+            os.mkdir(os.path.join(os.environ['XDG_CONFIG_HOME'], 'astropy'))
+            os.mkdir(os.path.join(os.environ['XDG_CACHE_HOME'], 'astropy'))
+
+            try:
+                retcode = subprocess.call([sys.executable, '-c', cmd],
+                                          cwd=testing_path, close_fds=False)
+            finally:
+                # kill the temporary dirs
+                shutil.rmtree(os.environ['XDG_CONFIG_HOME'])
+                shutil.rmtree(os.environ['XDG_CACHE_HOME'])
+
+            if self.coverage and retcode == 0:
+                # Copy the htmlcov from build/lib.../htmlcov to a more
+                # obvious place
+                if os.path.exists('htmlcov'):
+                    shutil.rmtree('htmlcov')
+                shutil.copytree(os.path.join(testing_path, 'htmlcov'), 'htmlcov')
+
+        finally:
+
+            # Remove temporary directory
+            shutil.rmtree(tmp_dir)
 
         raise SystemExit(retcode)
 
