@@ -7,9 +7,6 @@ packages, but this subpackage is for utilities that are primarily of use for
 developers or to implement python hacks. This subpackage also includes the
 `astropy.utils.compat` package, which houses utilities that provide
 compatibility and bugfixes across all versions of Python that Astropy supports.
-
-For astronomy-specific utilities of general use (e.g. not specific to some
-other subpackage), see the `astropy.tools` package.
 """
 
 from .compat.odict import OrderedDict
