@@ -7,13 +7,20 @@ from __future__ import division, print_function
 
 import re
 import math
+import multiprocessing
 import sys
+import threading
 import time
 
 try:
-    from IPython.zmq.iostream import OutStream
-except ImportError:
+    get_ipython()
+except NameError:
     OutStream = None
+else:
+    try:
+        from IPython.zmq.iostream import OutStream
+    except ImportError:
+        OutStream = None
 
 from ..config import ConfigurationItem
 
@@ -39,6 +46,10 @@ def isatty(file):
     but some user-defined types may not, so this assumes those are not
     ttys.
     """
+    if (multiprocessing.current_process().name != 'MainProcess' or
+        threading.current_thread().getName() != 'MainThread'):
+        return False
+
     if (OutStream is not None and
         isinstance(file, OutStream) and
         file.name == 'stdout'):
@@ -47,11 +58,54 @@ def isatty(file):
         return file.isatty()
     return False
 
+def _color_text(text, color):
+    """
+    Returns a string wrapped in ANSI color codes for coloring the
+    text in a terminal::
+
+        colored_text = color_text('Here is a message', 'blue')
+
+    This won't actually effect the text until it is printed to the
+    terminal.
+
+    Parameters
+    ----------
+    text : str
+        The string to return, bounded by the color codes.
+    color : str
+        An ANSI terminal color name. Must be one of:
+        black, red, green, brown, blue, magenta, cyan, lightgrey,
+        default, darkgrey, lightred, lightgreen, yellow, lightblue,
+        lightmagenta, lightcyan, white, or '' (the empty string).
+    """
+    color_mapping = {
+        'black': '0;30',
+        'red': '0;31',
+        'green': '0;32',
+        'brown': '0;33',
+        'blue': '0;34',
+        'magenta': '0;35',
+        'cyan': '0;36',
+        'lightgrey': '0;37',
+        'default': '0;39',
+        'darkgrey': '1;30',
+        'lightred': '1;31',
+        'lightgreen': '1;32',
+        'yellow': '1;33',
+        'lightblue': '1;34',
+        'lightmagenta': '1;35',
+        'lightcyan': '1;36',
+        'white': '1;37'}
+
+    color_code = color_mapping.get(color, '0;39')
+    return u'\033[{0}m{1}\033[0m'.format(color_code, text)
 
 def color_print(*args, **kwargs):
     """
     Prints colors and styles to the terminal uses ANSI escape
-    sequences::
+    sequences.
+
+    ::
 
        color_print('This is the color ', 'default', 'GREEN', 'green')
 
@@ -76,24 +130,6 @@ def color_print(*args, **kwargs):
         The ending of the message.  Defaults to ``\\n``.  The end will
         be printed after resetting any color or font state.
     """
-    color_mapping = {
-        'black': '0;30',
-        'red': '0;31',
-        'green': '0;32',
-        'brown': '0;33',
-        'blue': '0;34',
-        'magenta': '0;35',
-        'cyan': '0;36',
-        'lightgrey': '0;37',
-        'default': '0;39',
-        'darkgrey': '1;30',
-        'lightred': '1;31',
-        'lightgreen': '1;32',
-        'yellow': '1;33',
-        'lightblue': '1;34',
-        'lightmagenta': '1;35',
-        'lightcyan': '1;36',
-        'white': '1;37'}
 
     file = kwargs.get('file', sys.stdout)
     end = kwargs.get('end', u'\n')
@@ -113,9 +149,7 @@ def color_print(*args, **kwargs):
             if color == u'' or color is None:
                 write(msg)
             else:
-                color_code = color_mapping.get(color, '0;39')
-                write(u'\033[{0}m{1}\033[0m'.format(
-                    color_code, msg))
+                write(_color_text(msg, color))
 
         write(end)
     else:
@@ -126,7 +160,6 @@ def color_print(*args, **kwargs):
             write(msg)
         write(end)
 
-
 def strip_ansi_codes(s):
     """
     Remove ANSI color codes from the string.
@@ -298,7 +331,9 @@ class ProgressBar:
     def map(cls, function, items, multiprocess=False, file=sys.stdout):
         """
         Does a `map` operation while displaying a progress bar with
-        percentage complete::
+        percentage complete.
+
+        ::
 
             def work(i):
                 print(i)
@@ -324,6 +359,8 @@ class ProgressBar:
             calling its `isatty` member, if any), the scrollbar will
             be completely silent.
         """
+        results = []
+
         with cls(len(items), file=file) as bar:
             step_size = max(200, bar._bar_length)
             steps = max(int(float(len(items)) / step_size), 1)
@@ -335,15 +372,20 @@ class ProgressBar:
             else:
                 import multiprocessing
                 p = multiprocessing.Pool()
-                for i, _ in enumerate(
+                for i, result in enumerate(
                     p.imap_unordered(function, items, steps)):
                     bar.update(i)
+                    results.append(result)
+
+        return results
 
     @classmethod
     def iterate(cls, items, file=sys.stdout):
         """
         Iterate over a sequence while indicating progress with a progress
-        bar in the terminal::
+        bar in the terminal.
+
+        ::
 
             for item in ProgressBar.iterate(items):
                 pass
