@@ -1,19 +1,22 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """
-A "grab bag" of smallish general-purpose utilities that don't have a
-clear other module/pakcage to live in.
+A "grab bag" of relatively small general-purpose utilities that don't have
+a clear module/package to live in.
 """
 
 from __future__ import absolute_import
 
 import collections
+import contextlib
 import functools
 import sys
 import textwrap
+import traceback
 import warnings
 
-__all__ = ['find_current_module', 'fnpickle', 'fnunpickle', 'isiterable',
-           'deprecated', 'lazyproperty']
+__all__ = ['find_current_module', 'isiterable', 'deprecated', 'lazyproperty',
+           'deprecated_attribute', 'silence', 'format_exception',
+           'NumpyRNGContext']
 
 
 def find_current_module(depth=1, finddiff=False):
@@ -193,117 +196,6 @@ def find_mod_objs(modname, onlylocals=False):
     return localnames, fqnames, objs
 
 
-def fnunpickle(fileorname, number=0, usecPickle=True):
-    """ Unpickle pickled objects from a specified file and return the contents.
-
-    Parameters
-    ----------
-    fileorname : str or `file`-like
-        The file name or file from which to unpickle objects. If a file object,
-        it should have been opened in binary mode.
-    number : int
-        If 0, a single object will be returned (the first in the file). If >0,
-        this specifies the number of objects to be unpickled, and a list will
-        be returned with exactly that many objects. If <0, all objects in the
-        file will be unpickled and returned as a list.
-    usecPickle : bool
-        If True, the :mod:`cPickle` module is to be used in place of
-        :mod:`pickle` (cPickle is faster). This only applies for python 2.x.
-
-    Raises
-    ------
-    EOFError
-        If `number` is >0 and there are fewer than `number` objects in the
-        pickled file.
-
-    Returns
-    -------
-    contents : obj or list
-        If `number` is 0, this is a individual object - the first one unpickled
-        from the file. Otherwise, it is a list of objects unpickled from the
-        file.
-
-    """
-
-    if usecPickle and sys.version_info[0] < 3:  # pragma: py2
-        import cPickle as pickle
-    else:
-        import pickle
-
-    if isinstance(fileorname, basestring):
-        f = open(fileorname, 'rb')
-        close = True
-    else:
-        f = fileorname
-        close = False
-
-    try:
-        if number > 0:  # get that number
-            res = []
-            for i in range(number):
-                res.append(pickle.load(f))
-        elif number < 0:  # get all objects
-            res = []
-            eof = False
-            while not eof:
-                try:
-                    res.append(pickle.load(f))
-                except EOFError:
-                    eof = True
-        else:  # number==0
-            res = pickle.load(f)
-    finally:
-        if close:
-            f.close()
-
-    return res
-
-
-def fnpickle(object, fileorname, usecPickle=True, protocol=None, append=False):
-    """Pickle an object to a specified file.
-
-    Parameters
-    ----------
-    object
-        The python object to pickle.
-    fileorname : str or `file`-like
-        The filename or file into which the `object` should be pickled. If a
-        file object, it should have been opened in binary mode.
-    usecPickle : bool
-        If True (default), the :mod:`cPickle` module is to be used in place of
-        :mod:`pickle` (cPickle is faster). This only applies for python 2.x.
-    protocol : int or None
-        Pickle protocol to use - see the :mod:`pickle` module for details on
-        these options. If None, the most recent protocol will be used.
-    append : bool
-        If True, the object is appended to the end of the file, otherwise the
-        file will be overwritten (if a file object is given instead of a
-        file name, this has no effect).
-
-    """
-
-    if usecPickle and sys.version_info[0] < 3:  # pragma: py2
-        import cPickle as pickle
-    else:
-        import pickle
-
-    if protocol is None:
-        protocol = pickle.HIGHEST_PROTOCOL
-
-    if isinstance(fileorname, basestring):
-        f = open(fileorname, 'ab' if append else 'wb')
-        close = True
-    else:
-        f = fileorname
-        close = False
-
-    try:
-        pickle.dump(object, f, protocol=protocol)
-    finally:
-        if close:
-            f.close()
-
-
 def isiterable(obj):
     """Returns `True` if the given object is iterable."""
     from numpy import ndarray
@@ -335,7 +227,7 @@ class lazyproperty(object):
         >>> class LazyTest(object):
         ...     @lazyproperty
         ...     def complicated_property(self):
-        ...         print 'Computing the value for complicated_property..."
+        ...         print 'Computing the value for complicated_property...'
         ...         return 42
         ...
         >>> lt = LazyTest()
@@ -408,29 +300,33 @@ class lazyproperty(object):
 
 
 # TODO: Provide a class deprecation marker as well.
-def deprecated(since, message='', name='', alternative='', pending=False):
+def deprecated(since, message='', name='', alternative='', pending=False,
+               obj_type='function'):
     """
     Used to mark a function as deprecated.
 
-    To mark an attribute as deprecated, replace that attribute with a
-    depcrecated property.
+    To mark an attribute as deprecated, use `deprecated_attribute`.
 
     Parameters
     ------------
     since : str
-        The release at which this API became deprecated.  This is required.
+        The release at which this API became deprecated.  This is
+        required.
 
     message : str, optional
-        Override the default deprecation message.  The format specifier
-        %(func)s may be used for the name of the function, and %(alternative)s
-        may be used in the deprecation message to insert the name of an
-        alternative to the deprecated function.
+        Override the default deprecation message.  The format
+        specifier `%(func)s` may be used for the name of the function,
+        and `%(alternative)s` may be used in the deprecation message
+        to insert the name of an alternative to the deprecated
+        function.  `%(obj_type)` may be used to insert a friendly name
+        for the type of object being deprecated.
 
     name : str, optional
-        The name of the deprecated function; if not provided the name is
-        automatically determined from the passed in function, though this is
-        useful in the case of renamed functions, where the new function is just
-        assigned to the name of the deprecated function.  For example::
+        The name of the deprecated function; if not provided the name
+        is automatically determined from the passed in function,
+        though this is useful in the case of renamed functions, where
+        the new function is just assigned to the name of the
+        deprecated function.  For example::
 
             def new_function():
                 ...
@@ -473,16 +369,20 @@ def deprecated(since, message='', name='', alternative='', pending=False):
         altmessage = ''
         if not message or type(message) == type(deprecate):
             if pending:
-                message = ('The %(func)s function will be deprecated in a '
+                message = ('The %(func)s %(obj_type)s will be deprecated in a '
                            'future version.')
             else:
-                message = ('The %(func)s function is deprecated and may '
+                message = ('The %(func)s %(obj_type)s is deprecated and may '
                            'be removed in a future version.')
             if alternative:
                 altmessage = '\n        Use %s instead.' % alternative
 
-        message = ((message % {'func': name, 'alternative': alternative}) +
-                   altmessage)
+        message = ((message % {
+            'func': name,
+            'name': name,
+            'alternative': alternative,
+            'obj_type': obj_type}) +
+            altmessage)
 
         @functools.wraps(func)
         def deprecated_func(*args, **kwargs):
@@ -520,3 +420,167 @@ def deprecated(since, message='', name='', alternative='', pending=False):
         return deprecate(message)
 
     return deprecate
+
+
+def deprecated_attribute(name, since, message=None, alternative=None,
+                         pending=False):
+    """
+    Used to mark a public attribute as deprecated.  This creates a
+    property that will warn when the given attribute name is accessed.
+    To prevent the warning (i.e. for internal code), use the private
+    name for the attribute by prepending an underscore
+    (i.e. `self._name`).
+
+    Parameters
+    ----------
+    name : str
+        The name of the deprecated attribute.
+
+    since : str
+        The release at which this API became deprecated.  This is
+        required.
+
+    message : str, optional
+        Override the default deprecation message.  The format
+        specifier `%(name)s` may be used for the name of the attribute,
+        and `%(alternative)s` may be used in the deprecation message
+        to insert the name of an alternative to the deprecated
+        function.
+
+    alternative : str, optional
+        An alternative attribute that the user may use in place of the
+        deprecated attribute.  The deprecation warning will tell the
+        user about this alternative if provided.
+
+    pending : bool, optional
+        If True, uses a PendingDeprecationWarning instead of a
+        DeprecationWarning.
+
+    Examples
+    --------
+
+    ::
+
+        class MyClass:
+            # Mark the old_name as deprecated
+            old_name = misc.deprecated_attribute('old_name', '0.1')
+
+            def method(self):
+                self._old_name = 42
+    """
+    private_name = '_' + name
+
+    @deprecated(since, name=name, obj_type='attribute')
+    def get(self):
+        return getattr(self, private_name)
+
+    @deprecated(since, name=name, obj_type='attribute')
+    def set(self, val):
+        setattr(self, private_name, val)
+
+    @deprecated(since, name=name, obj_type='attribute')
+    def delete(self):
+        delattr(self, private_name)
+
+    return property(get, set, delete)
+
+
+class _DummyFile(object):
+    """A noop writeable object."""
+
+    def write(self, s):
+        pass
+
+
+@contextlib.contextmanager
+def silence():
+    """A context manager that silences sys.stdout and sys.stderr."""
+
+    old_stdout = sys.stdout
+    old_stderr = sys.stderr
+    sys.stdout = _DummyFile()
+    sys.stderr = _DummyFile()
+    yield
+    sys.stdout = old_stdout
+    sys.stderr = old_stderr
+
+
+def format_exception(msg, *args, **kwargs):
+    """
+    Given an exception message string, uses new-style formatting arguments
+    ``{filename}``, ``{lineno}``, ``{func}`` and/or ``{text}`` to fill in
+    information about the exception that occurred.  For example:
+
+        try:
+            1/0
+        except:
+            raise ZeroDivisionError(
+                format_except('A divide by zero occurred in {filename} at '
+                              'line {lineno} of function {func}.'))
+
+    Any additional positional or keyword arguments passed to this function are
+    also used to format the message.
+
+    .. note::
+        This uses `sys.exc_info` to gather up the information needed to
+        fill in the formatting arguments. Python 2.x and 3.x have slightly
+        different behavior regarding `sys.exc_info` (the latter will not carry
+        it outside a handled exception), so it's not wise to use this outside of
+        an `except` clause - if it is, this will substitute '<unkown>' for the 4
+        formatting arguments.
+    """
+
+    tb = traceback.extract_tb(sys.exc_info()[2], limit=1)
+    if len(tb) > 0:
+        filename, lineno, func, text = tb[0]
+    else:
+        filename = lineno = func = text = '<unknown>'
+
+    return msg.format(*args, filename=filename, lineno=lineno, func=func,
+                      text=text, **kwargs)
+
+
+class NumpyRNGContext(object):
+    """
+    A context manager (for use with the ``with`` statement) that will seed the
+    numpy random number generator (RNG) to a specific value, and then restore
+    the RNG state back to whatever it was before.
+
+    This is primarily intended for use in the astropy testing suit, but it
+    may be useful in ensuring reproducibility of Monte Carlo simulations in a
+    science context.
+
+    Parameters
+    ----------
+    seed : int
+        The value to use to seed the numpy RNG
+
+    Examples
+    --------
+    A typical use case might be::
+
+        with NumpyRNGContext(<some seed value you pick>):
+            from numpy import random
+
+            randarr = random.randn(100)
+            ... run your test using `randarr` ...
+
+        #Any code using numpy.random at this indent level will act just as it
+        #would have if it had been before the with statement - e.g. whatever
+        #the default seed is.
+
+
+    """
+    def __init__(self, seed):
+        self.seed = seed
+
+    def __enter__(self):
+        from numpy import random
+
+        self.startstate = random.get_state()
+        random.seed(self.seed)
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        from numpy import random
+
+        random.set_state(self.startstate)
