@@ -1,3 +1,4 @@
+# Licensed under a 3-clause BSD style license - see LICENSE.rst
 # -*- coding: utf-8 -*-
 
 import io
@@ -7,6 +8,8 @@ from astropy.tests.helper import raises
 
 from .. import console
 
+def test_color_text():
+    assert console._color_text("foo", "green") == u'\033[0;32mfoo\033[0m'
 
 def test_color_print():
     # This stuff is hard to test, at least smoke test it
