@@ -11,8 +11,8 @@
  *     faster and more memory efficient than the alternatives in the
  *     Python standard library because it does not build a tree of
  *     objects, and also throws away most text nodes, since for
- *     astropy.io.vo (the primary user of this library) we only care
- *     about simple text nodes contained between a single pair of
+ *     astropy.io.votable (the primary user of this library) we only
+ *     care about simple text nodes contained between a single pair of
  *     open/close element nodes.  It also has an optimization for
  *     recognizing the most commonly occuring element in a VO file,
  *     "TD".
@@ -338,7 +338,19 @@ startElement(IterParser *self, const XML_Char *name, const XML_Char **atts)
             }
             do {
                 if (*(*(att_ptr + 1)) != 0) {
+                    /* Python < 2.6.5 can't handle unicode keyword
+                       arguments.  Since those were coming from here
+                       (the dictionary of attributes), we use byte
+                       strings for the keys instead.  Should be fine
+                       for VOTable, since it has ascii attribute
+                       names, but that's not true of XML in
+                       general. */
+                    #if PY_VERSION_HEX < 0x02060500
+                    /* Due to Python issue #4978 */
+                    key = PyBytes_FromString(*att_ptr);
+                    #else
                     key = PyUnicode_FromString(*att_ptr);
+                    #endif
                     if (key == NULL) {
                         Py_DECREF(tuple);
                         XML_StopParser(self->parser, 0);
@@ -927,7 +939,12 @@ IterParser_init(IterParser *self, PyObject *args, PyObject *kwds)
     }
     text_clear(self);
 
+    #if PY_VERSION_HEX < 0x02060500
+    /* Due to Python issue #7249 */
+    self->read_args = PyTuple_Pack(1, PyInt_FromSsize_t(buffersize));
+    #else
     self->read_args = PyTuple_Pack(1, PyLong_FromSsize_t(buffersize));
+    #endif
     if (self->read_args == NULL) {
         goto fail;
     }
