@@ -1,13 +1,17 @@
+# Licensed under a 3-clause BSD style license - see LICENSE.rst
 import glob
 import os
 import sys
 import warnings
 
 import numpy as np
-from numpy.testing import assert_array_almost_equal
+from numpy.testing import (
+    assert_array_almost_equal, assert_array_almost_equal_nulp)
 
+from ...tests.helper import raises
 from ... import wcs
-from ...config import get_data_filenames, get_data_contents, get_data_filename
+from ...utils.data import (
+    get_pkg_data_filenames, get_pkg_data_contents, get_pkg_data_filename)
 from ...tests.helper import pytest
 
 
@@ -19,7 +23,8 @@ def test_maps():
 
         # the test parameter is the base name of the file to use; find
         # the file in the installed wcs test directory
-        header = get_data_contents(os.path.join("maps", filename))
+        header = get_pkg_data_contents(
+            os.path.join("maps", filename), encoding='binary')
         wcsobj = wcs.WCS(header)
 
         world = wcsobj.wcs_pix2world([[97, 97]], 1)
@@ -31,7 +36,7 @@ def test_maps():
         assert_array_almost_equal(pix, [[97, 97]], decimal=0)
 
     # get the list of the hdr files that we want to test
-    hdr_file_list = list(get_data_filenames("maps", "*.hdr"))
+    hdr_file_list = list(get_pkg_data_filenames("maps", "*.hdr"))
 
     # actually perform a test for each one
     for filename in hdr_file_list:
@@ -69,7 +74,8 @@ def test_spectra():
 
         # the test parameter is the base name of the file to use; find
         # the file in the installed wcs test directory
-        header = get_data_contents(os.path.join("spectra", filename))
+        header = get_pkg_data_contents(
+            os.path.join("spectra", filename), encoding='binary')
 
         wcsobj = wcs.WCS(header)
 
@@ -77,7 +83,7 @@ def test_spectra():
         assert len(all) == 9
 
     # get the list of the hdr files that we want to test
-    hdr_file_list = list(get_data_filenames("spectra", "*.hdr"))
+    hdr_file_list = list(get_pkg_data_filenames("spectra", "*.hdr"))
 
     # actually perform a test for each one
     for filename in hdr_file_list:
@@ -115,8 +121,8 @@ derived_units = "Hz J W V N Pa C Ohm ohm S F Wb T H lm lx".split()
 add_all_units = "eV Jy R G barn".split()
 add_sup_units = "a yr pc bit byte Byte".split()
 add_sub_units = "mag".split()
-general_units = \
-              "deg arcmin arcsec mas d h min erg Ry u D DEGREE DEGREES".split()
+general_units = (
+    "deg arcmin arcsec mas d h min erg Ry u D DEGREE DEGREES".split())
 astro_units = "Angstrom angstrom AU lyr beam solRad solMass solLum Sun".split()
 device_units = "adu bin chan count ct photon ph pixel pix voxel".split()
 
@@ -148,7 +154,7 @@ def test_all_units():
 
     # list of all the units to test
     all = sorted(basic_units + derived_units + add_all_units + add_sup_units
-            + add_sub_units + general_units + astro_units + device_units)
+                 + add_sub_units + general_units + astro_units + device_units)
 
     # Pandokia has non-case-sensitve test names; since the unit name is
     # showing up in the test name, we want to disambiguate any name collisions.
@@ -206,13 +212,14 @@ def test_fixes():
     From github issue #36
     """
     def run():
-        header = get_data_contents('data/nonstandard_units.hdr')
+        header = get_pkg_data_contents(
+            'data/nonstandard_units.hdr', encoding='binary')
         w = wcs.WCS(header)
 
     with warnings.catch_warnings(record=True) as w:
         warnings.simplefilter("always")
         run()
-        assert len(w) == 3
+        assert len(w) == 2
         for item in w:
             assert issubclass(item.category, wcs.FITSFixedWarning)
             if 'unitfix' in str(item.message):
@@ -223,23 +230,132 @@ def test_outside_sky():
     """
     From github issue #107
     """
-    header = get_data_contents('data/outside_sky.hdr')
+    header = get_pkg_data_contents(
+        'data/outside_sky.hdr', encoding='binary')
     w = wcs.WCS(header)
 
-    assert np.all(np.isnan(w.wcs_pix2world([[100.,500.]], 0)))  # outside sky
-    assert np.all(np.isnan(w.wcs_pix2world([[200.,200.]], 0)))  # outside sky
-    assert not np.any(np.isnan(w.wcs_pix2world([[1000.,1000.]], 0)))
+    assert np.all(np.isnan(w.wcs_pix2world([[100., 500.]], 0)))  # outside sky
+    assert np.all(np.isnan(w.wcs_pix2world([[200., 200.]], 0)))  # outside sky
+    assert not np.any(np.isnan(w.wcs_pix2world([[1000., 1000.]], 0)))
 
 
 def test_load_fits_path():
-    fits = get_data_filename('data/sip.fits')
+    fits = get_pkg_data_filename('data/sip.fits')
     w = wcs.WCS(fits)
 
 
 def test_backward_compatible():
-    fits = get_data_filename('data/sip.fits')
+    fits = get_pkg_data_filename('data/sip.fits')
     w = wcs.WCS(fits)
 
     data = np.random.rand(100, 2)
     assert np.all(w.wcs_pix2world(data, 0) == w.wcs_pix2sky(data, 0))
     assert np.all(w.wcs_world2pix(data, 0) == w.wcs_sky2pix(data, 0))
+
+
+def test_dict_init():
+    """
+    Test that WCS can be initialized with a dict-like object
+    """
+
+    # Dictionary with no actual WCS, returns identity transform
+    w = wcs.WCS({})
+
+    xp, yp = w.wcs_world2pix(41., 2., 1)
+
+    assert_array_almost_equal_nulp(xp, 41., 10)
+    assert_array_almost_equal_nulp(yp, 2., 10)
+
+    # Valid WCS
+    w = wcs.WCS({'CTYPE1': 'GLON-CAR',
+                 'CTYPE2': 'GLAT-CAR',
+                 'CUNIT1': 'deg',
+                 'CUNIT2': 'deg',
+                 'CRPIX1': 1,
+                 'CRPIX2': 1,
+                 'CRVAL1': 40.,
+                 'CRVAL2': 0.,
+                 'CDELT1': -0.1,
+                 'CDELT2': 0.1})
+
+    xp, yp = w.wcs_world2pix(41., 2., 0)
+
+    assert_array_almost_equal_nulp(xp, -10., 10)
+    assert_array_almost_equal_nulp(yp, 20., 10)
+
+
+@raises(TypeError)
+def test_extra_kwarg():
+    """
+    Issue #444
+    """
+    w = wcs.WCS()
+    data = np.random.rand(100, 2)
+    w.wcs_pix2sky(data, origin=1)
+
+
+def test_3d_shapes():
+    """
+    Issue #444
+    """
+    w = wcs.WCS(naxis=3)
+    data = np.random.rand(100, 3)
+    result = w.wcs_pix2sky(data, 1)
+    assert result.shape == (100, 3)
+    result = w.wcs_pix2sky(
+        data[..., 0], data[..., 1], data[..., 2], 1)
+    assert len(result) == 3
+
+
+def test_preserve_shape():
+    w = wcs.WCS(naxis=2)
+
+    x = np.random.random((2,3,4))
+    y = np.random.random((2,3,4))
+
+    xw, yw = w.wcs_pix2world(x, y, 1)
+
+    assert xw.shape == (2,3,4)
+    assert yw.shape == (2,3,4)
+
+    xp, yp = w.wcs_world2pix(x, y, 1)
+
+    assert xp.shape == (2,3,4)
+    assert yp.shape == (2,3,4)
+
+
+def test_broadcasting():
+    w = wcs.WCS(naxis=2)
+
+    x = np.random.random((2,3,4))
+    y = 1
+
+    xp, yp = w.wcs_world2pix(x, y, 1)
+
+    assert xp.shape == (2,3,4)
+    assert yp.shape == (2,3,4)
+
+
+def test_shape_mismatch():
+    w = wcs.WCS(naxis=2)
+
+    x = np.random.random((2,3,4))
+    y = np.random.random((3,2,4))
+
+    with pytest.raises(ValueError) as exc:
+        xw, yw = w.wcs_pix2world(x, y, 1)
+    assert exc.value.args[0] == "Coordinate arrays are not broadcastable to each other"
+
+    with pytest.raises(ValueError) as exc:
+        xp, yp = w.wcs_world2pix(x, y, 1)
+    assert exc.value.args[0] == "Coordinate arrays are not broadcastable to each other"
+
+@raises(wcs._wcs.InvalidTransformError)
+def test_find_all_wcs_crash():
+    """
+    Causes a double free without a recent fix in wcslib_wrap.C
+    """
+    from ...io import fits
+
+    with fits.open(get_pkg_data_filename("data/too_many_pv.hdr")) as hdulist:
+        wcses = wcs.find_all_wcs(hdulist[0].header)
