@@ -1,3 +1,4 @@
+# Licensed under a 3-clause BSD style license - see LICENSE.rst
 """
 Under the hood, there are 3 separate classes that perform different
 parts of the transformation:
@@ -45,12 +46,13 @@ try:
     from . import _wcs
 except ImportError:
     _wcs = None
-from ..utils import deprecated
+from ..utils import deprecated, deprecated_attribute
+from .. import log
 
 if _wcs is not None:
     assert _wcs._sanity_check(), \
-           "astropy.wcs did not pass its sanity check for your build " \
-           "on your platform."
+        "astropy.wcs did not pass its sanity check for your build " \
+        "on your platform."
 
 if sys.version_info[0] >= 3:  # pragma: py3
     string_types = (bytes, str)
@@ -68,7 +70,7 @@ if _wcs is not None:
     DistortionLookupTable = _wcs.DistortionLookupTable
     Sip = _wcs.Sip
     UnitConverter = _wcs.UnitConverter
-    Wcsprm = _wcs._Wcsprm
+    Wcsprm = _wcs.Wcsprm
     Tabprm = _wcs.Tabprm
     # Copy all the constants from the C extension into this module's namespace
     for key, val in _wcs.__dict__.items():
@@ -77,6 +79,10 @@ if _wcs is not None:
             key.startswith('WCSHDO')):
             locals()[key] = val
             __all__.append(key)
+
+    UnitConverter = deprecated(
+        '0.2', name='UnitConverter', alternative='astropy.units')(
+            UnitConverter)
 else:
     WCSBase = object
     Wcsprm = object
@@ -86,6 +92,10 @@ else:
     Tabprm = object
 
 
+# Additional relax bit flags
+WCSHDO_SIP = 0x10000
+
+
 def _parse_keysel(keysel):
     keysel_flags = 0
     if keysel is not None:
@@ -122,7 +132,7 @@ class WCS(WCSBase):
 
     Parameters
     ----------
-    header : astropy.io.fits header object, string or None, optional
+    header : astropy.io.fits header object, string, dict-like, or None, optional
         If *header* is not provided or None, the object will be
         initialized to default values.
 
@@ -144,12 +154,12 @@ class WCS(WCSBase):
     relax : bool or int, optional
         Degree of permissiveness:
 
+        - `True` (default): Admit all recognized informal extensions
+          of the WCS standard.
+
         - `False`: Recognize only FITS keywords defined by the
           published WCS standard.
 
-        - `True`: Admit all recognized informal extensions of the WCS
-          standard.
-
         - `int`: a bit field selecting specific extensions to accept.
           See :ref:`relaxread` for details.
 
@@ -210,24 +220,52 @@ class WCS(WCSBase):
 
     Notes
     -----
-    astropy.wcs supports arbitrary *n* dimensions for the core WCS
-    (the transformations handled by WCSLIB).  However, the Paper IV
-    lookup table and SIP distortions must be two dimensional.
-    Therefore, if you try to create a WCS object where the core WCS
-    has a different number of dimensions than 2 and that object also
-    contains a Paper IV lookup table or SIP distortion, a `ValueError`
-    exception will be raised.  To avoid this, consider using the
-    *naxis* kwarg to select two dimensions from the core WCS.
+
+    1. astropy.wcs supports arbitrary *n* dimensions for the core WCS
+       (the transformations handled by WCSLIB).  However, the Paper IV
+       lookup table and SIP distortions must be two dimensional.
+       Therefore, if you try to create a WCS object where the core WCS
+       has a different number of dimensions than 2 and that object
+       also contains a Paper IV lookup table or SIP distortion, a
+       `ValueError` exception will be raised.  To avoid this, consider
+       using the *naxis* kwarg to select two dimensions from the core
+       WCS.
+
+    2. The number of coordinate axes in the transformation is not
+       determined directly from the ``NAXIS`` keyword but instead from
+       the highest of:
+
+           - ``NAXIS`` keyword
+
+           - ``WCSAXESa`` keyword
+
+           - The highest axis number in any parameterized WCS keyword.
+             The keyvalue, as well as the keyword, must be
+             syntactically valid otherwise it will not be considered.
+
+       If none of these keyword types is present, i.e. if the header
+       only contains auxiliary WCS keywords for a particular
+       coordinate representation, then no coordinate description is
+       constructed for it.
+
+       The number of axes, which is set as the `naxis` member, may
+       differ for different coordinate representations of the same
+       image.
+
+    3. When the header includes duplicate keywords, in most cases the
+       last encountered is used.
     """
 
     def __init__(self, header=None, fobj=None, key=' ', minerr=0.0,
-                 relax=False, naxis=None, keysel=None, colsel=None,
+                 relax=True, naxis=None, keysel=None, colsel=None,
                  fix=True):
+        close_fds = []
+
         if header is None:
             if naxis is None:
                 naxis = 2
-            wcsprm = _wcs._Wcsprm(header=None, key=key,
-                                  relax=relax, naxis=naxis)
+            wcsprm = _wcs.Wcsprm(header=None, key=key,
+                                 relax=relax, naxis=naxis)
             self.naxis = wcsprm.naxis
             # Set some reasonable defaults.
             det2im = (None, None)
@@ -243,28 +281,44 @@ class WCS(WCSBase):
                             "Can not provide both a FITS filename to "
                             "argument 1 and a FITS file object to argument 2")
                     fobj = fits.open(header)
+                    close_fds.append(fobj)
                     header = fobj[0].header
-                    header_string = repr(header.ascard).encode('latin1')
+                    header_string = header.tostring()
                 else:
                     header_string = header
             elif isinstance(header, fits.Header):
-                header_string = repr(header.ascard).encode('latin1')
+                header_string = header.tostring()
             else:
-                raise TypeError(
-                    "header must be a string or an astropy.io.fits.Header "
-                    "object")
+                try:
+                    # Accept any dict-like object
+                    new_header = fits.Header()
+                    for dict_key in header:
+                        new_header[dict_key] = header[dict_key]
+                    header_string = new_header.tostring()
+                except TypeError:
+                    raise TypeError(
+                        "header must be a string, an astropy.io.fits.Header "
+                        "object, or a dict-like object")
+
+            if isinstance(header_string, unicode):
+                header_bytes = header_string.encode('ascii')
+                header_string = header_string
+            else:
+                header_bytes = header_string
+                header_string = header_string.decode('ascii')
+
             try:
-                wcsprm = _wcs._Wcsprm(header=header_string, key=key,
-                                      relax=relax, keysel=keysel_flags,
-                                      colsel=colsel)
+                wcsprm = _wcs.Wcsprm(header=header_bytes, key=key,
+                                     relax=relax, keysel=keysel_flags,
+                                     colsel=colsel)
             except _wcs.NoWcsKeywordsFoundError:
                 # The header may have SIP or distortions, but no core
                 # WCS.  That isn't an error -- we want a "default"
                 # (identity) core Wcs transformation in that case.
                 if colsel is None:
-                    wcsprm = _wcs._Wcsprm(header=None, key=key,
-                                          relax=relax, keysel=keysel_flags,
-                                          colsel=colsel)
+                    wcsprm = _wcs.Wcsprm(header=None, key=key,
+                                         relax=relax, keysel=keysel_flags,
+                                         colsel=colsel)
                 else:
                     raise
 
@@ -272,7 +326,9 @@ class WCS(WCSBase):
                 wcsprm = wcsprm.sub(naxis)
             self.naxis = wcsprm.naxis
 
-            det2im = self._read_det2im_kw(header, fobj)
+            header = fits.Header.fromstring(header_string)
+
+            det2im = self._read_det2im_kw(header, fobj, err=minerr)
             cpdis = self._read_distortion_kw(
                 header, fobj, dist='CPDIS', err=minerr)
             sip = self._read_sip_kw(header)
@@ -287,6 +343,13 @@ distortion, you must select or reduce these to 2 dimensions using the
 naxis kwarg.
 """.format(wcsprm.naxis))
 
+            header_naxis = header.get('NAXIS', None)
+            if header_naxis is not None and header_naxis < wcsprm.naxis:
+                log.info(
+                    "The WCS transformation has more axes ({0:d}) than the "
+                    "image it is associated with ({1:d})".format(
+                        wcsprm.naxis, header_naxis))
+
         if fix:
             fixes = wcsprm.fix()
             for key, val in fixes.iteritems():
@@ -297,9 +360,12 @@ naxis kwarg.
                         format(key, val),
                         FITSFixedWarning)
 
-        self.get_naxis(header)
+        self._get_naxis(header)
         WCSBase.__init__(self, sip, cpdis, wcsprm, det2im)
 
+        for fd in close_fds:
+            fd.close()
+
     def __copy__(self):
         new_copy = self.__class__()
         WCSBase.__init__(new_copy, self.sip,
@@ -347,9 +413,9 @@ naxis kwarg.
         copy.naxis = copy.wcs.naxis
         return copy
     if _wcs is not None:
-        sub.__doc__ = _wcs._Wcsprm.sub.__doc__
+        sub.__doc__ = _wcs.Wcsprm.sub.__doc__
 
-    def calcFootprint(self, header=None, undistort=True):
+    def calcFootprint(self, header=None, undistort=True, axes=None):
         """
         Calculates the footprint of the image on the sky.
 
@@ -365,22 +431,31 @@ naxis kwarg.
             If `True`, take SIP and distortion lookup table into
             account
 
+        axes : length 2 sequence ints, optional
+            If provided, use the given sequence as the shape of the
+            image.  Otherwise, use the ``NAXIS1`` and ``NAXIS2``
+            keywords from the header that was used to create this
+            `WCS` object.
+
         Returns
         -------
         coord : (4, 2) array of (*x*, *y*) coordinates.
         """
-        if header is None:
-            try:
-                # classes that inherit from WCS and define naxis1/2
-                # do not require a header parameter
-                naxis1 = self.naxis1
-                naxis2 = self.naxis2
-            except AttributeError:
-                print("Need a valid header in order to calculate footprint\n")
-                return None
+        if axes is not None:
+            naxis1, naxis2 = axes
         else:
-            naxis1 = header.get('NAXIS1', None)
-            naxis2 = header.get('NAXIS2', None)
+            if header is None:
+                try:
+                    # classes that inherit from WCS and define naxis1/2
+                    # do not require a header parameter
+                    naxis1 = self._naxis1
+                    naxis2 = self._naxis2
+                except AttributeError:
+                    print("Need a valid header in order to calculate footprint\n")
+                    return None
+            else:
+                naxis1 = header.get('NAXIS1', None)
+                naxis2 = header.get('NAXIS2', None)
 
         corners = np.zeros(shape=(4, 2), dtype=np.float64)
         if naxis1 is None or naxis2 is None:
@@ -399,7 +474,7 @@ naxis kwarg.
         else:
             return self.wcs_pix2world(corners, 1)
 
-    def _read_det2im_kw(self, header, fobj):
+    def _read_det2im_kw(self, header, fobj, err=0.0):
         """
         Create a `Paper IV`_ type lookup table for detector to image
         plane correction.
@@ -415,6 +490,10 @@ naxis kwarg.
         if not isinstance(fobj, fits.HDUList):
             return (None, None)
 
+        d_error = header.get('D2IMERR', 0.0)
+        if d_error < err:
+            return (None, None)
+
         try:
             d2im_data = fobj[('D2IMARR', 1)].data
         except KeyError:
@@ -547,21 +626,46 @@ naxis kwarg.
             if cpdis is None:
                 return
 
-            hdulist[0].header.update('{0}{1:d}'.format(dist, num), 'LOOKUP')
-            hdulist[0].header.update('{0}{1:d}.EXTVER'.format(d_kw, num),
-                                     len(hdulist))
-            hdulist[0].header.update('{0}{1:d}.AXIS.{1:d}'.format(d_kw, num),
-                                     num)
+            hdulist[0].header.update(
+                '{0}{1:d}'.format(dist, num),
+                ('LOOKUP', 'Prior distortion function type'))
+            hdulist[0].header.update(
+                '{0}{1:d}.EXTVER'.format(d_kw, num),
+                (num, 'Version number of WCSDVARR extension'))
+            hdulist[0].header.update(
+                '{0}{1:d}.NAXES'.format(d_kw, num),
+                (len(cpdis.data.shape),
+                 'Number of independent variables in distortion function'))
+
+            for i in range(cpdis.data.ndim):
+                hdulist[0].header.update(
+                    '{0}{1:d}.AXIS.{2:d}'.format(d_kw, num, i + 1),
+                    (i + 1, 'Axis number of the jth independent variable in a '
+                     'distortion function'))
 
             image = fits.ImageHDU(cpdis.data, name='WCSDVARR')
             header = image.header
 
-            header.update('CRPIX1', cpdis.crpix[0])
-            header.update('CRPIX2', cpdis.crpix[1])
-            header.update('CRVAL1', cpdis.crval[0])
-            header.update('CRVAL2', cpdis.crval[1])
-            header.update('CDELT1', cpdis.cdelt[0])
-            header.update('CDELT2', cpdis.cdelt[1])
+            header.update(
+                'CRPIX1',
+                (cpdis.crpix[0], 'Coordinate system reference pixel'))
+            header.update(
+                'CRPIX2',
+                (cpdis.crpix[1], 'Coordinate system reference pixel'))
+            header.update(
+                'CRVAL1',
+                (cpdis.crval[0], 'Coordinate system value at reference pixel'))
+            header.update(
+                'CRVAL2',
+                (cpdis.crval[1], 'Coordinate system value at reference pixel'))
+            header.update(
+                'CDELT1',
+                (cpdis.cdelt[0], 'Coordinate increment along axis'))
+            header.update(
+                'CDELT2',
+                (cpdis.cdelt[1], 'Coordinate increment along axis'))
+            image.update_ext_version(
+                int(hdulist[0].header['{0}{1:d}.EXTVER'.format(d_kw, num)]))
 
             hdulist.append(image)
 
@@ -659,7 +763,8 @@ naxis kwarg.
             for i in range(size):
                 for j in range(size - i):
                     if a[i, j] != 0.0:
-                        keywords['{0}_{1:d}_{2:d}'.format(name, i, j)] = a[i, j]
+                        keywords[
+                            '{0}_{1:d}_{2:d}'.format(name, i, j)] = a[i, j]
 
         write_array('A', self.sip.a)
         write_array('B', self.sip.b)
@@ -735,7 +840,11 @@ naxis kwarg.
         A helper function to support reading either a pair of arrays
         or a single Nx2 array.
         """
-        ra_dec_order = kwargs.get('ra_dec_order')
+        ra_dec_order = kwargs.pop('ra_dec_order', False)
+        if len(kwargs):
+            raise TypeError("Unexpected keyword argument {0!r}".format(
+                kwargs.keys()[0]))
+
         if len(args) == 2:
             xy, origin = args
             try:
@@ -743,37 +852,46 @@ naxis kwarg.
                 origin = int(origin)
             except:
                 raise TypeError(
-                    "When providing two arguments, they must be (xy, origin)")
+                    "When providing two arguments, they must be "
+                    "(coords[N][{0}], origin)".format(self.naxis))
             if ra_dec_order and sky == 'input':
                 xy = self._denormalize_sky(xy)
             result = func(xy, origin)
             if ra_dec_order and sky == 'output':
                 result = self._normalize_sky(result)
             return result
-        elif len(args) == 3:
-            x, y, origin = args
+        elif len(args) == self.naxis + 1:
+            axes = args[:-1]
+            origin = args[-1]
             try:
-                x = np.asarray(x)
-                y = np.asarray(y)
+                axes = [np.asarray(x) for x in axes]
                 origin = int(origin)
             except:
                 raise TypeError(
-                    "When providing three arguments, they must be " +
-                    "(x, y, origin)")
-            if x.size != y.size:
-                raise ValueError("x and y arrays are not the same size")
-            length = x.size
-            xy = np.hstack((x.reshape((length, 1)),
-                            y.reshape((length, 1))))
+                    "When providing more than two arguments, they must be " +
+                    "a 1-D array for each axis, followed by an origin.")
+
+            try:
+                axes = np.broadcast_arrays(*axes)
+            except ValueError:
+                raise ValueError(
+                    "Coordinate arrays are not broadcastable to each other")
+
+            xy = np.hstack([x.reshape((x.size, 1)) for x in axes])
+
             if ra_dec_order and sky == 'input':
                 xy = self._denormalize_sky(xy)
             sky = func(xy, origin)
             if ra_dec_order and sky == 'output':
                 sky = self._normalize_sky_output(sky)
-                return sky[:, 0], sky[:, 1]
-            return [sky[:, i] for i in range(sky.shape[1])]
+                return (sky[:, 0].reshape(axes[0].shape),
+                        sky[:, 1].reshape(axes[0].shape))
+            return [sky[:, i].reshape(axes[0].shape)
+                    for i in range(sky.shape[1])]
+
         raise TypeError(
-            "Expected 2 or 3 arguments, {0} given".format(len(args)))
+            "Expected 2 or {0} arguments, {0} given".format(
+                self.naxis + 1, len(args)))
 
     def all_pix2world(self, *args, **kwargs):
         return self._array_converter(
@@ -839,7 +957,7 @@ naxis kwarg.
 
         InvalidTransformError
             Ill-conditioned coordinate transformation parameters.
-        """.format(__.TWO_OR_THREE_ARGS('naxis', 8),
+        """.format(__.TWO_OR_MORE_ARGS('naxis', 8),
                    __.RA_DEC_ORDER(8),
                    __.RETURNS('sky coordinates, in degrees', 8))
 
@@ -912,7 +1030,7 @@ naxis kwarg.
         `~astropy.wcs.Wcsprm.lattyp` and `~astropy.wcs.Wcsprm.lngtyp`
         members can be used to determine the order of the axes.
 
-        """.format(__.TWO_OR_THREE_ARGS('naxis', 8),
+        """.format(__.TWO_OR_MORE_ARGS('naxis', 8),
                    __.RA_DEC_ORDER(8),
                    __.RETURNS('world coordinates, in degrees', 8))
 
@@ -979,7 +1097,7 @@ naxis kwarg.
 
         InvalidTransformError
             Ill-conditioned coordinate transformation parameters.
-        """.format(__.TWO_OR_THREE_ARGS('naxis', 8),
+        """.format(__.TWO_OR_MORE_ARGS('naxis', 8),
                    __.RA_DEC_ORDER(8),
                    __.RETURNS('pixel coordinates', 8))
 
@@ -987,8 +1105,8 @@ naxis kwarg.
     def wcs_sky2pix(self, *args, **kwargs):
         return self.wcs_world2pix(*args, **kwargs)
 
-    def pix2foc(self, *args, **kwargs):
-        return self._array_converter(self._pix2foc, None, *args, **kwargs)
+    def pix2foc(self, *args):
+        return self._array_converter(self._pix2foc, None, *args)
     pix2foc.__doc__ = """
         Convert pixel coordinates to focal plane coordinates using the
         `SIP`_ polynomial distortion convention and `Paper IV`_
@@ -1011,11 +1129,11 @@ naxis kwarg.
 
         ValueError
             Invalid coordinate transformation parameters.
-        """.format(__.TWO_OR_THREE_ARGS('2', 8),
+        """.format(__.TWO_OR_MORE_ARGS('2', 8),
                    __.RETURNS('focal coordinates', 8))
 
-    def p4_pix2foc(self, *args, **kwargs):
-        return self._array_converter(self._p4_pix2foc, None, *args, **kwargs)
+    def p4_pix2foc(self, *args):
+        return self._array_converter(self._p4_pix2foc, None, *args)
     p4_pix2foc.__doc__ = """
         Convert pixel coordinates to focal plane coordinates using
         `Paper IV`_ table-lookup distortion correction.
@@ -1037,11 +1155,11 @@ naxis kwarg.
 
         ValueError
             Invalid coordinate transformation parameters.
-        """.format(__.TWO_OR_THREE_ARGS('2', 8),
+        """.format(__.TWO_OR_MORE_ARGS('2', 8),
                    __.RETURNS('focal coordinates', 8))
 
-    def det2im(self, *args, **kwargs):
-        return self._array_converter(self._det2im, None, *args, **kwargs)
+    def det2im(self, *args):
+        return self._array_converter(self._det2im, None, *args)
     det2im.__doc__ = """
         Convert detector coordinates to image plane coordinates using
         `Paper IV`_ table-lookup distortion correction.
@@ -1063,10 +1181,10 @@ naxis kwarg.
 
         ValueError
             Invalid coordinate transformation parameters.
-        """.format(__.TWO_OR_THREE_ARGS('2', 8),
+        """.format(__.TWO_OR_MORE_ARGS('2', 8),
                    __.RETURNS('pixel coordinates', 8))
 
-    def sip_pix2foc(self, *args, **kwargs):
+    def sip_pix2foc(self, *args):
         if self.sip is None:
             if len(args) == 2:
                 return args[0]
@@ -1074,7 +1192,7 @@ naxis kwarg.
                 return args[:2]
             else:
                 raise TypeError("Wrong number of arguments")
-        return self._array_converter(self.sip.pix2foc, None, *args, **kwargs)
+        return self._array_converter(self.sip.pix2foc, None, *args)
     sip_pix2foc.__doc__ = """
         Convert pixel coordinates to focal plane coordinates using the
         `SIP`_ polynomial distortion convention.
@@ -1102,10 +1220,10 @@ naxis kwarg.
 
         ValueError
             Invalid coordinate transformation parameters.
-        """.format(__.TWO_OR_THREE_ARGS('2', 8),
+        """.format(__.TWO_OR_MORE_ARGS('2', 8),
                    __.RETURNS('focal coordinates', 8))
 
-    def sip_foc2pix(self, *args, **kwargs):
+    def sip_foc2pix(self, *args):
         if self.sip is None:
             if len(args) == 2:
                 return args[0]
@@ -1113,7 +1231,7 @@ naxis kwarg.
                 return args[:2]
             else:
                 raise TypeError("Wrong number of arguments")
-        return self._array_converter(self.sip.foc2pix, None, *args, **kwargs)
+        return self._array_converter(self.sip.foc2pix, None, *args)
     sip_foc2pix.__doc__ = """
         Convert focal plane coordinates to pixel coordinates using the
         `SIP`_ polynomial distortion convention.
@@ -1139,7 +1257,7 @@ naxis kwarg.
 
         ValueError
             Invalid coordinate transformation parameters.
-        """.format(__.TWO_OR_THREE_ARGS('2', 8),
+        """.format(__.TWO_OR_MORE_ARGS('2', 8),
                    __.RETURNS('pixel coordinates', 8))
 
     def to_fits(self, relax=False):
@@ -1156,10 +1274,10 @@ naxis kwarg.
         relax : bool or int, optional
             Degree of permissiveness:
 
-            - `False`: Recognize only FITS keywords defined by the
-              published WCS standard.
+            - `False` (default): Write all extensions that are
+              considered to be safe and recommended.
 
-            - `True`: Admit all recognized informal extensions of the
+            - `True`: Write all recognized informal extensions of the
               WCS standard.
 
             - `int`: a bit field selecting specific extensions to
@@ -1199,10 +1317,10 @@ naxis kwarg.
         relax : bool or int, optional
             Degree of permissiveness:
 
-            - `False`: Recognize only FITS keywords defined by the
-              published WCS standard.
+            - `False` (default): Write all extensions that are
+              considered to be safe and recommended.
 
-            - `True`: Admit all recognized informal extensions of the
+            - `True`: Write all recognized informal extensions of the
               WCS standard.
 
             - `int`: a bit field selecting specific extensions to
@@ -1245,13 +1363,19 @@ naxis kwarg.
           8. Keyword order may be changed.
         """
 
+        do_sip = (relax is True or
+                  relax == WCSHDO_all or
+                  (relax & WCSHDO_SIP))
+        if relax not in (True, False):
+            relax &= ~WCSHDO_SIP
+
         if self.wcs is not None:
             header_string = self.wcs.to_header(relax)
             header = fits.Header.fromstring(header_string)
         else:
             header = fits.Header()
 
-        if self.sip is not None:
+        if do_sip and self.sip is not None:
             for key, val in self._write_sip_kw().items():
                 header.update(key, val)
 
@@ -1295,12 +1419,19 @@ naxis kwarg.
         f.write(') # color={0}, width={1:d} \n'.format(color, width))
         f.close()
 
+    naxis1 = deprecated_attribute('naxis1', '0.2')
+    naxis2 = deprecated_attribute('naxis2', '0.2')
+
+    @deprecated('0.2', message='This method should not be public')
     def get_naxis(self, header=None):
-        self.naxis1 = 0.0
-        self.naxis2 = 0.0
-        if header != None and not isinstance(header, string_types):
-            self.naxis1 = header.get('NAXIS1', 0.0)
-            self.naxis2 = header.get('NAXIS2', 0.0)
+        return self._get_naxis(header=header)
+
+    def _get_naxis(self, header=None):
+        self._naxis1 = 0
+        self._naxis2 = 0
+        if header is not None and not isinstance(header, string_types):
+            self.naxis1 = header.get('NAXIS1', 0)
+            self.naxis2 = header.get('NAXIS2', 0)
 
     def rotateCD(self, theta):
         _theta = np.deg2rad(theta)
@@ -1329,7 +1460,7 @@ naxis kwarg.
 
     def get_axis_types(self):
         """
-        Similar to `self.wcsprm.axis_types <_wcs._Wcsprm.axis_types>`
+        Similar to `self.wcsprm.axis_types <_wcs.Wcsprm.axis_types>`
         but provides the information in a more Python-friendly format.
 
         Returns
@@ -1455,7 +1586,7 @@ def __WCS_unpickle__(cls, dct, fits_data):
     return self
 
 
-def find_all_wcs(header, relax=False, keysel=None):
+def find_all_wcs(header, relax=True, keysel=None):
     """
     Find all the WCS transformations in the given header.
 
@@ -1466,12 +1597,12 @@ def find_all_wcs(header, relax=False, keysel=None):
     relax : bool or int, optional
         Degree of permissiveness:
 
+        - `True` (default): Admit all recognized informal extensions of the
+          WCS standard.
+
         - `False`: Recognize only FITS keywords defined by the
           published WCS standard.
 
-        - `True`: Admit all recognized informal extensions of the
-          WCS standard.
-
         - `int`: a bit field selecting specific extensions to accept.
           See :ref:`relaxread` for details.
 
@@ -1503,14 +1634,19 @@ def find_all_wcs(header, relax=False, keysel=None):
     if isinstance(header, string_types):
         header_string = header
     elif isinstance(header, fits.Header):
-        header_string = repr(header)
+        header_string = header.tostring()
     else:
         raise TypeError(
             "header must be a string or astropy.io.fits.Header object")
 
     keysel_flags = _parse_keysel(keysel)
 
-    wcsprms = _wcs.find_all_wcs(header_string, relax, keysel_flags)
+    if isinstance(header_string, unicode):
+        header_bytes = header_string.encode('ascii')
+    else:
+        header_bytes = header_string
+
+    wcsprms = _wcs.find_all_wcs(header_bytes, relax, keysel_flags)
 
     result = []
     for wcsprm in wcsprms:
